﻿using System;
using System.Linq;
using Grpc.Net.Client;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc.Testing;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using PXPlus.Core.IntegrationEventLogEf;
using PXPlus.Payment.Api;
using PXPlus.Payment.Infrastructure;
using PXPlus.Payment.IntegrationTest.MockServices;

namespace PXPlus.Payment.IntegrationTest
{
    public class TestServerFixture 
    {
        public CustomWebApplicationFactory<Startup> Factory;
        public GrpcChannel GrpcChannel { get; }
        public IServiceProvider Services { get; }
        public TestServerFixture()
        {
            Factory = new CustomWebApplicationFactory<Startup>();
            
            var client = Factory.CreateDefaultClient();

            GrpcChannel = GrpcChannel.ForAddress(client.BaseAddress, new GrpcChannelOptions
            {
                HttpClient = client
            });

            Services = Factory.Services;

        }
        
        public void Dispose()
        {
            Factory.Dispose();
        }
    }

    public class CustomWebApplicationFactory<TStartup> : WebApplicationFactory<TStartup> where TStartup : class
    {
        protected override void ConfigureWebHost(IWebHostBuilder builder)
        {
            builder.ConfigureServices(services =>
            {
                var descriptor = services.SingleOrDefault(
                    d => d.ServiceType ==
                         typeof(DbContextOptions<PaymentContext>));

                services.Remove(descriptor);

                var connStr = "Server=192.168.249.85;Database=pxplus_payment_for_integration_test;User Id=sa;Password=P@ssw0rd;";
                
                services.AddDbContext<PaymentContext>(options =>
                {
                    options.UseSqlServer(connStr);
                });

                services.AddDbContext<IntegrationEventLogContext>(options =>
                {
                    options.UseSqlServer(connStr);
                });
                
                var sp = services.BuildServiceProvider();

                using var scope = sp.CreateScope();
                var scopedServices = scope.ServiceProvider;
                var paymentContext = scopedServices.GetRequiredService<PaymentContext>();

                paymentContext.Database.EnsureCreated();
                paymentContext.Database.Migrate();
                
                var integrationEvent = scopedServices.GetRequiredService<IntegrationEventLogContext>();

                integrationEvent.Database.EnsureCreated();

                services.RegisterMockService();
            });
        }
    }
}