# Payment Integration Events

## OrderPaymentSuccessfulIntegrationEvent
-  Topic : order_payment_successful

## OrderPaymentFailedIntegrationEvent
- Topic: order_payment_failed

### Model

Name | Type | Require | Description | Notes
------------ | ------------- | ------------- | ------------- | -------------
**PayerId** | **long** | **Y** | **買家MemberId** |
**OrderNo** | **string** | **Y** | **付款訂單號** |
**OrderAmount** | **decimal** | **Y** | **訂單金額**|
**PayAmount** | **decimal** | **Y** | **支付金額**|
**CashBack** | **decimal** | **Y** | **現金回饋**|
**PointBack** | **decimal** | **Y** | **點數回饋**|
**PayerPaymentAccount** | **string** | **Y** | **買家付款工具帳號** |
**PayerPayToolType** | **int** | **Y** | **買家付款工具類別** |
**StoreMemberId** | **long** | **Y** | **收款店家MemberId** |
**MerchantCode** | **string** | **Y** | **收款店家代碼** |
**MerchantName** | **string** | **Y** | **收款店家名稱** |
**StoreCode** | **string** | **Y** | **收款店鋪代碼** |
**StoreName** | **string** | **Y** | **收款店鋪名稱** |
**StoreReceiveAccount** | **string** | **Y** | **收款店家帳號** |
**Fees** | **Fee[]** | **Y** | **手續費** |
**ActualDiscountAmount** | **decimal** | **Y** | **實際折抵點數** |
**TransactionTime** | **DateTime** | **Y** | **交易時間** |
**MerchantTradeNo** | **string** | **Y** | **店家訂單號** |
**CurrencyType** | **int** | **Y** | **1 = NTD** |
**PhoneNumber** | **string** | **Y** | **會員電話** |
**BrandName** | **string** | **Y** | **品牌名稱** |
**InputMetaData** | **MetaData** | **N** | **上游MetaData** |
**ReceiveMetaData** | **MetaData** | **N** | **下游MetaData** |
**BankCode** | **string** | **N** | **銀行代碼** |
**BankName** | **string** | **N** | **銀行名稱** |
**BankReturnCode** | **string** | **N** | **銀行回應碼** |
**Note** | **string** | **N** | **備註** |


class:
```c#
    public long PayerId { get; set; }
    public string OrderNo { get; set; }
    public decimal OrderAmount { get; set; }
    public decimal PayAmount { get; set; }
    public decimal CashBack { get; set; }
    public decimal PointBack { get; set; }
    public string PayerPaymentAccount { get; set; }
    public int PayerPayToolType { get; set; }
    public long StoreMemberId { get; set; }
    public string MerchantCode { get; set; }
    public string MerchantName { get; set; }
    public string StoreCode { get; set; }
    public string StoreName { get; set; }
    public string StoreReceiveAccount { get; set; }
    public decimal ActualDiscountAmount { get; set; }
    public DateTime TransactionTime { get; set; }
    public IEnumerable<Fee> Fees {get;set;}
    public string MerchantTradeNo { get; set; }
    public CurrencyType CurrencyType { get; set; }
    public string PhoneNumber { get; set; }
    public string BrandName { get; set; }
    public MetaData InputMetaData { get; set; }
    public MetaData ReceiveMetaData { get; set; }
    public string BankCode { get; set; }
    public string BankName { get; set; }
    public string BankReturnCode { get; set; }
    public string Note { get; set; }
```

Example:

```json
{
  "PayerId":20,
  "OrderNo":"PX202103221056216093",  
  "OrderAmount":100,
  "PayAmount":93,
  "CashBack": 1,
  "PointBack": 2,
  "PayerPaymentAccount" : "1234567890",
  "PayerPayToolType" : 1,
  "StoreMemberId":111,
  "MerchantCode":"HelloMerchant",
  "MerchantName":"HelloMerchantName",
  "StoreCode":"HelloStore",
  "StoreName":"HelloStoreName",
  "StoreReceiveAccount": "123456789000",
  "ActualDiscountAmount":7,
  "TransactionTime" : "2021-04-29T10:00:00",
  "Fees": [
    {
      "Type" : 1,
      "Unit" : 1,
      "Rate" : 10
    },{
      "Type" : 2,
      "Unit" : 2,
      "Rate" : 0.05
    }
  ],
  "MerchantTradeNo" :"HelloMerchantTradeNo",
  "CurrencyType" : 1,
  "PhoneNumber" : "HelloPhoneNumber",
  "BrandName" : "HelloBrandName",
  "InputMetaData": {
    "Type": 1,
    "Content" : "HelloContent"
  },
  "ReceiveMetaData":{
    "Type": 1,
    "Content" : "HelloContent"
  },
  "BankCode": "HelloBankCode",
  "BankName" : "HelloBankName",
  "BankReturnCode" :"HelloBankReturnCode",
  "Note": "note"
}
```
#### Fee

Name | Type | Require | Description | Notes
------------ | ------------- | ------------- | ------------- | -------------
**Type** | **int** | **Y** | **1 = 支付手續費 2 = 點數手續費** |
**Unit** | **int** | **Y** | **滿額算 或 比率** |
**Rate** | **decimal** | **Y** | **若滿額 則直接扣 若比率 直接乘**

## OrderRefundSuccessfulIntegrationEvent
- Topic: order_refund_successful

## OrderRefundFailedIntegrationEvent
- Topic: order_refund_failed

### Model

Name | Type | Require | Description | Notes
------------ | ------------- | ------------- | ------------- | -------------
**OrderNo** | **string** | **Y** | **原單號** |
**RefundOrderNo** | **string** | **Y** | **退單號** |
**OrderAmount** | **decimal** | **Y** | **退款金額** |
**PayAmount** | **decimal** | **Y** | **實際退款金額** |
**CashBack** | **decimal** | **Y** | **扣回現金回饋**|
**PointBack** | **decimal** | **Y** | **扣回點數回饋**|
**PayerId** | **long** | **Y** | **付款人ID** |
**MerchantCode** | **string** | **Y** | **商戶** |
**StoreCode** | **string** | **Y** | **商店** |
**StoreMemberId** | **long** | **Y** | **商戶會員ID** |
**ReturnDiscountPointAmount** | **decimal** | **Y** | **退還點數金額** |
**TransactionTime** | **DateTime** | **Y** | **交易時間** |
**MerchantTradeNo** | **string** | **Y** | **店家訂單號** |
**RefundCause** | **string** | **Y** | **退款原因** |
**ExecuteSource** | **int** | **Y** | **執行來源** |
**Executor** | **string** | **Y** | **執行人** |

Class:

```c#
    public string OrderNo { get; set; }
    public string RefundOrderNo { get; set; }  
    public decimal OrderAmount { get; set; }    
    public decimal PayAmount { get; set; }
    public decimal CashBack { get; set; }
    public decimal PointBack { get; set; }
    public long PayerId { get; set; }
    public string MerchantCode { get; set; }
    public string StoreCode { get; set; }
    public long StoreMemberId { get; set; }    
    public decimal ReturnDiscountPointAmount { get; set; }
    public DateTime TransactionTime { get; set; }
    public string MerchantTradeNo { get; set; }
    public string RefundCause { get; set; }
    public int ExecuteSource { get; set; }
    public string Executor { get; set; }    
```

Example:

```json
{
  "OrderNo": "PX2021052514302359",
  "RefundOrderNo": "PX20210525143023590001",
  "OrderAmount" : 100,
  "PayAmount" : 80,
  "CashBack": 1,
  "PointBack" : 1,
  "PayerId":20,
  "MerchantCode":"vv",
  "StoreCode": "bvb",
  "StoreMemberId": 12,  
  "ReturnDiscountPointAmount":10,  
  "TransactionTime" : "2021-04-29T10:00:00",
  "MerchantTradeNo" : "d",
  "RefundCause": "a",
  "ExecuteSource": "b",
  "Executor" : "c"
}
```

