using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;

namespace PXPlus.Payment.Application.Behaviors
{
    public class LoggingBehavior<TRequest, TResponse> : IPipelineBehavior<TRequest, TResponse>
    {
        private readonly ILogger<LoggingBehavior<TRequest, TResponse>> _logger;
        private readonly IHttpContextAccessor _httpContextAccessor;
        
        public LoggingBehavior(ILogger<LoggingBehavior<TRequest, TResponse>> logger,
            IHttpContextAccessor httpContextAccessor)
        {
            _logger = logger;
            _httpContextAccessor = httpContextAccessor;
        }

        public async Task<TResponse> Handle(TRequest request, CancellationToken cancellationToken,
            RequestHandlerDelegate<TResponse> next)
        {
            var requestId = _httpContextAccessor.HttpContext.TraceIdentifier;

            if (string.IsNullOrEmpty(requestId))
                requestId = Guid.NewGuid().ToString();

            using (_logger.BeginScope(new[] { new KeyValuePair<string, object>("requestid", requestId)}))
            {
                _logger.LogInformation("----- Handling command {CommandName} ({@Command})", typeof(TRequest).Name, request);

                var response = await next();

                _logger.LogInformation("----- Command {CommandName} handled - response: {@Response}",
                    typeof(TRequest).Name,response);
                return response;
            }
        }
    }
}