﻿using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.AspNetCore.Http;
using PXPlus.Cache.Abstract;
using PXPlus.Payment.Application.Dto;

namespace PXPlus.Payment.Application.Commands
{
    public class CancelPaymentCommandHandler : IRequestHandler<CancelPaymentCommand,Response>
    {
        private readonly ICacheLinkList _cacheLinkList;
        private readonly IHttpContextAccessor _httpContextAccessor;

        public CancelPaymentCommandHandler(ICacheLinkList cacheLinkList, IHttpContextAccessor httpContextAccessor)
        {
            _cacheLinkList = cacheLinkList;
            _httpContextAccessor = httpContextAccessor;
        }
        
        public async Task<Response> Handle(CancelPaymentCommand request, CancellationToken cancellationToken)
        {
            request.XRequestId = _httpContextAccessor.HttpContext.TraceIdentifier;
            
            await _cacheLinkList.LPushAsync("ps_payment_cancel", JsonSerializer.Serialize(request));

            return new Response
            {
                Code = "0000",
                Message = "成功",
                Prefix = "PS"
            };
        }
    }
}