﻿using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.AspNetCore.Http;
using PXPlus.Core.IntegrationEventLogEf.Services;
using PXPlus.Payment.Application.IntegrationEvents.Events;
using PXPlus.Payment.Domain.Events;
using PXPlus.Payment.Domain.OrderRecordAggregate;

namespace PXPlus.Payment.Application.DomainEventHandlers.OrderPayment
{
    public class OrderPaymentFailedDomainEventHandler : INotificationHandler<OrderPaymentFailedDomainEvent>

    {
        private readonly IOrderRecordRepository _orderRecordRepository;
        private readonly IIntegrationEventService _integrationEventService;
        private readonly IHttpContextAccessor _httpContextAccessor;

        public OrderPaymentFailedDomainEventHandler(IOrderRecordRepository orderRecordRepository, IIntegrationEventService integrationEventService, IHttpContextAccessor httpContextAccessor)
        {
            _orderRecordRepository = orderRecordRepository;
            _integrationEventService = integrationEventService;
            _httpContextAccessor = httpContextAccessor;
        }

        public async Task Handle(OrderPaymentFailedDomainEvent notification, CancellationToken cancellationToken)
        {
            var orderRecord = OrderRecord.FromOrderDomainModelForPaymentOrder(notification.Order);
            orderRecord.SetRecordTypeAndResult(OrderRecordResult.Failed);

            await _orderRecordRepository.AddAsync(orderRecord);
            await _orderRecordRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);

            
            var xRequestId = _httpContextAccessor.HttpContext.TraceIdentifier;

            var order = notification.Order;
            var orderPaymentFailedIntegrationEvent =
                new OrderPaymentFailedIntegrationEvent(order,xRequestId);
            
            var paymentRecordIntegrationEvent = new PaymentRecordIntegrationEvent(
                orderPaymentFailedIntegrationEvent, order.PayerInfo.PayerId, xRequestId);
            
            await _integrationEventService.AddAndSaveEventAsync(orderPaymentFailedIntegrationEvent);
            await _integrationEventService.AddAndSaveEventAsync(paymentRecordIntegrationEvent);
        }
    }
}