﻿using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.AspNetCore.Http;
using PXPlus.Core.IntegrationEventLogEf.Services;
using PXPlus.Payment.Application.IntegrationEvents.Events;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.Events;
using PXPlus.Payment.Domain.OrderRecordAggregate;

namespace PXPlus.Payment.Application.DomainEventHandlers.OrderPayment
{
    public class OrderPaymentSuccessfulDomainEventHandler : INotificationHandler<OrderPaymentSuccessfulDomainEvent>
    {
        private readonly IOrderRecordRepository _orderRecordRepository;
        private readonly IActivityDomainService _activityDomainService;
        private readonly IIntegrationEventService _integrationEventService;
        private readonly IHttpContextAccessor _httpContextAccessor;
        public OrderPaymentSuccessfulDomainEventHandler(IOrderRecordRepository orderRecordRepository,
            IActivityDomainService activityDomainService, IIntegrationEventService integrationEventService
            ,IHttpContextAccessor httpContextAccessor)
        {
            _orderRecordRepository = orderRecordRepository;
            _activityDomainService = activityDomainService;
            _integrationEventService = integrationEventService;
            _httpContextAccessor = httpContextAccessor;
        }

        public async Task Handle(OrderPaymentSuccessfulDomainEvent notification, CancellationToken cancellationToken)
        {
            var order = notification.Order;

            //活動現金回饋  
            var activityFeedbackInfos =
                await _activityDomainService.CalculateActivitiesFeedbackAsync(order.PayerInfo.PayerId);

            order.AddActivityFeedbackInfo(activityFeedbackInfos);

            var orderRecord = OrderRecord.FromOrderDomainModelForPaymentOrder(notification.Order);

            orderRecord.SetRecordTypeAndResult(OrderRecordResult.Success);

            await _orderRecordRepository.AddAsync(orderRecord);

            await _orderRecordRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);

            var requestId = _httpContextAccessor.HttpContext.TraceIdentifier;
            
            var orderPaymentSuccessfulIntegrationEvent = new OrderPaymentSuccessfulIntegrationEvent(order, requestId);

            var paymentRecordIntegrationEvent = new PaymentRecordIntegrationEvent(
                orderPaymentSuccessfulIntegrationEvent, order.PayerInfo.PayerId, requestId);

            await _integrationEventService.AddAndSaveEventAsync(orderPaymentSuccessfulIntegrationEvent);
            await _integrationEventService.AddAndSaveEventAsync(paymentRecordIntegrationEvent);
        }
    }
}