using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.AspNetCore.Http;
using Microsoft.EntityFrameworkCore;
using PXPlus.Core.IntegrationEventLogEf.Services;
using PXPlus.Payment.Application.IntegrationEvents.Events;
using PXPlus.Payment.Domain.Events;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderRecordAggregate;

namespace PXPlus.Payment.Application.DomainEventHandlers.OrderRefund
{
    public class OrderRefundFailedDomainEventHandler : INotificationHandler<OrderRefundFailedDomainEvent>
    {
        private readonly IOrderRecordRepository _orderRecordRepository;
        private readonly IIntegrationEventService _integrationEventService;
        private readonly IHttpContextAccessor _httpContextAccessor;
        private readonly IOrderRepository _orderRepository;

        public OrderRefundFailedDomainEventHandler(IOrderRecordRepository orderRecordRepository, IIntegrationEventService integrationEventService, IHttpContextAccessor httpContextAccessor,
            IOrderRepository orderRepository)
        {
            _orderRecordRepository = orderRecordRepository;
            _integrationEventService = integrationEventService;
            _httpContextAccessor = httpContextAccessor;
            _orderRepository = orderRepository;
        }

        public async Task Handle(OrderRefundFailedDomainEvent notification, CancellationToken cancellationToken)
        {
            //Rollback order
            _orderRepository.UnChangeOrder(notification.Order);

            var orderRecord = OrderRecord.FromOrderDomainModelForRefundOrder(notification.Order, notification.Order.RefundOrderNo.GetOrderNo()
                , 0, 0, 0,0,0);

            orderRecord.SetRecordTypeAndResult(OrderRecordResult.Failed);

            await _orderRecordRepository.AddAsync(orderRecord);
            
            await _orderRecordRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);
            
            var xRequestId =  _httpContextAccessor.HttpContext.TraceIdentifier;
            var orderRefundFailedIntegrationEvent = new OrderRefundFailedIntegrationEvent(notification.Order,orderRecord,xRequestId);
            
            var paymentRecordIntegrationEvent = new PaymentRecordIntegrationEvent(
                    orderRefundFailedIntegrationEvent, notification.Order.PayerInfo.PayerId, xRequestId);
            
            await _integrationEventService.AddAndSaveEventAsync(orderRefundFailedIntegrationEvent);
            await _integrationEventService.AddAndSaveEventAsync(paymentRecordIntegrationEvent);
        }
    }
}