﻿using System;
using System.Collections.Generic;
using PXPlus.Core.EventBus.Events;
using PXPlus.Payment.Domain.OrderAggregate;

namespace PXPlus.Payment.Application.IntegrationEvents.Events
{
    public record OrderPaymentFailedIntegrationEvent : IntegrationEvent,IOrderPaymentIntegrationEvent
    {
        public decimal PointBack { get; set; }
        public long PayerId { get; set; }
        public string OrderNo { get; set; }
        public decimal OrderAmount { get; set; }
        public decimal PayAmount { get; set; }
        public decimal CashBack { get; set; }
        public string PayerPaymentAccount { get; set; }
        public int PayerPayToolType { get; set; }
        public long StoreMemberId { get; set; }
        public string MerchantCode { get; set; }
        public string MerchantName { get; set; }
        public string StoreCode { get; set; }
        public string StoreName { get; set; }
        public string StoreReceiveAccount { get; set; }
        public decimal ActualDiscountAmount { get; set; }
        public DateTime TransactionTime { get; set; }
        public string MerchantTradeNo { get; set; }
        public CurrencyType CurrencyType { get; set; }
        public string PhoneNumber { get; set; }
        public string BrandName { get; set; }
        public MetaData InputMetaData { get; set; }
        public MetaData ReceiveMetaData { get; set; }
        public string BankCode { get; set; }
        public string BankName { get; set; }
        public string BankReturnCode { get; set; }
        public string Note { get; set; }
        public string PayerEpAccount { get; set; }
        public IEnumerable<Fee> Fees { get; set; }

        public OrderPaymentFailedIntegrationEvent(Order order,string xRequestId):base(xRequestId,order.ReceiveStoreInfo.StoreCode)
        {
            OrderNo = order.OrderNo.GetOrderNo();
            OrderAmount = order.OrderAmount;
            PayAmount = order.PayAmount;
            CashBack = order.BackInfo.CashBack;
            PointBack = order.BackInfo.PointBack;
            PayerId = order.PayerInfo.PayerId;
            PayerPaymentAccount = order.PayerInfo.PayerPaymentAccount;
            PayerPayToolType = (int)order.PayerInfo.PayToolType;
            StoreMemberId = order.ReceiveStoreInfo.StoreMemberId;
            MerchantCode = order.ReceiveStoreInfo.MerchantCode;
            MerchantName = order.ReceiveStoreInfo.MerchantName;
            StoreCode = order.ReceiveStoreInfo.StoreCode;
            StoreName = order.ReceiveStoreInfo.StoreName;
            StoreReceiveAccount = order.ReceiveStoreInfo.ReceiveAccount;
            ActualDiscountAmount = order.PointDiscountInfo.ActualDiscountAmount;
            TransactionTime = order.GetCreatedTime;
            MerchantTradeNo = order.ReceiveStoreInfo.MerchantTradeNo;
            Fees = order.ReceiveStoreInfo.Fees;
            CurrencyType = order.CurrencyType;
            PhoneNumber = order.PayerInfo.PhoneNumber;
            BrandName = order.ReceiveStoreInfo.BrandName;
            InputMetaData = order.InputMetaData;
            ReceiveMetaData = order.ReceiveMetaData;
            BankCode = order.PayerInfo.GetPayTool().BankCode;
            BankName = order.PayerInfo.GetPayTool().BankName;
            BankReturnCode = order.BankReturnCode;
            Note = "Todo note";
            PayerEpAccount = order.PayerInfo.PayerEpAccount;
            
        }
        
        
        // for JsonConstructor
        public OrderPaymentFailedIntegrationEvent()
        {
            
        }
    }
}