using System;
using PXPlus.Core.EventBus.Events;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderRecordAggregate;

namespace PXPlus.Payment.Application.IntegrationEvents.Events
{
    public record OrderRefundSuccessfulIntegrationEvent : IntegrationEvent,IOrderRefundIntegrationEvent
    {
        public string OrderNo { get; set; }
        public string RefundOrderNo { get; set; }
        /// <summary>
        /// 退款金額
        /// </summary>
        public decimal OrderAmount { get; set; }
        /// <summary>
        /// 退還消費者金額
        /// </summary>
        public decimal PayAmount { get; set; }

        public decimal CashBack { get; set; }
        public decimal PointBack { get; set; }
        public long PayerId { get; set; }
        public string PayerPaymentAccount { get; set; }
        public int PayerPayToolType { get; set; }
        public string MerchantCode { get; set; }
        public string StoreCode { get; set; }
        public long StoreMemberId { get; set; }
        public string StoreReceiveAccount { get; set; }
        /// <summary>
        /// 退還消費者點數
        /// </summary>
        public decimal ReturnDiscountPointAmount { get; set; }
        public DateTime TransactionTime { get; set; }
        public string MerchantTradeNo { get; set; }
        public string RefundCause { get; set; }
        public int ExecuteSource { get; set; }
        public string Executor { get; set; }
        public string PhoneNumber { get; set; }
        public string BrandName { get; set; }
        public string MerchantName { get; set; }
        public string StoreName { get; set; }
        public CurrencyType CurrencyType { get; set; }
        public string BankCode { get; set; }
        public string BankName { get; set; }

        public OrderRefundSuccessfulIntegrationEvent(Order order,OrderRecord orderRecord,string xRequestId):base(xRequestId,order.ReceiveStoreInfo.StoreCode)
        {
            OrderNo = order.OrderNo.GetOrderNo();
            RefundOrderNo = order.RefundOrderNo.GetOrderNo();
            OrderAmount = orderRecord.OrderAmount;
            PayAmount = orderRecord.PayAmount;
            PayerId = order.PayerInfo.PayerId;
            CashBack = orderRecord.CashBack;
            PointBack = orderRecord.PointBack;
            PayerPaymentAccount = order.PayerInfo.PayerPaymentAccount;
            PayerPayToolType = (int)order.PayerInfo.PayToolType;
            MerchantCode = order.ReceiveStoreInfo.MerchantCode;
            StoreCode = order.ReceiveStoreInfo.StoreCode;
            StoreMemberId = order.ReceiveStoreInfo.StoreMemberId;
            StoreReceiveAccount = order.ReceiveStoreInfo.ReceiveAccount;
            ReturnDiscountPointAmount = orderRecord.ActualDiscountAmount;
            TransactionTime = order.GetUpdatedTime;
            MerchantTradeNo = order.ReceiveStoreInfo.MerchantTradeNo;
            RefundCause = order.RefundCause;
            ExecuteSource = order.ExecuteSource;
            Executor = order.Executor;
            PhoneNumber = order.PayerInfo.PhoneNumber;
            BrandName = order.ReceiveStoreInfo.BrandName;
            MerchantName = order.ReceiveStoreInfo.MerchantName;
            StoreName = order.ReceiveStoreInfo.StoreName;
            CurrencyType = order.CurrencyType;
            BankCode = order.PayerInfo.GetBankCode();
            BankName = order.PayerInfo.GetBankName();
        }
        
        // for JsonConstructor
        public OrderRefundSuccessfulIntegrationEvent()
        {
            
        }

        // public string OrderNo { get; set; }
        // public string RefundOrderNo { get; set; }
        // public decimal OrderAmount { get; set; }
        // public decimal PayAmount { get; set; }
        // public long PayerId { get; set; }
        // public string MerchantCode { get; set; }
        // public string StoreCode { get; set; }
        // public long StoreMemberId { get; set; }
        // public decimal ReturnDiscountPointAmount { get; set; }
        // public DateTime TransactionTime { get; set; }
        // public string RefundCause { get; set; }
        // public int ExecuteSource { get; set; }
        // public string Executor { get; set; }
        // public string MerchantTradeNo { get; set; }
    }
}