using System.Collections.Generic;
using PXPlus.Enum.Authorization;
using PXPlus.Payment.Domain.OrderAggregate;
using Xunit;

namespace PXPlus.Payment.Domain.Test.OrderAggregateTest
{
    public class OrderPaymentTest
    {
        private Order NewOrder(decimal orderAmount, decimal noneDiscountAmount, decimal noneFeedbackAmount,
            int pointDiscountPercent)
            => new Order(new OrderNo("a","a"),null, null, orderAmount, noneDiscountAmount,
                noneFeedbackAmount, new PointDiscountInfo(pointDiscountPercent, false), PlatformType.PXPlus,
                new List<ActivityCashDiscountInfo>());

        /// <summary>
        /// PayAmount = OrderAmount - PointDiscount
        ///    80     =    100      -      20
        /// </summary>
        [Fact]
        public void Order100Point20PayAmount80()
        {
            var order = NewOrder(100, 0, 0, 30);

            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(20);
            order.CalculatePaymentDiscount();

            Assert.Equal(80, order.PayAmount);
        }

        /// <summary>
        /// PayAmount = OrderAmount - PointDiscount - Activity
        ///    50     =    100      -      30       -    20
        /// </summary>
        [Fact]
        public void Order100Point30Activity20PayAmount50()
        {
            var order = NewOrder(100, 0, 0, 30);

            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(30);

            order.AddActivityCashDiscountInfo(new List<ActivityCashDiscountInfo>
            {
                new ActivityCashDiscountInfo("aa", 20)
                {
                    CashDiscountAmount = 20
                }
            });
            order.CalculatePaymentDiscount();

            Assert.Equal(50, order.PayAmount);
        }

        /// <summary>
        /// 折抵上限30% 付款101 上限101*0.3=[30.3]=30 折抵 = 28 pay amount = 101-28 = 73
        /// </summary>
        [Fact]
        public void OrderAmount101_PointDiscount28_AndFinalPayAmountIs73()
        {
            var order = NewOrder(101, 0, 0, 30);

            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(28);
            order.CalculatePaymentDiscount();

            Assert.Equal(73, order.PayAmount);
        }

        /// <summary>
        /// 折抵上限30% 付款101 不可折抵 90 上限11*0.3=[3.3]=3 折抵 = 3 pay amount = 98
        /// </summary>
        [Fact]
        public void OrderAmount101_NoneDiscount90_PointDiscount3_AndFinalPayAmountIs98()
        {
            var order = NewOrder(101, 90, 0, 30);

            order.CalculateAvailablePointDiscount();

            order.PointDiscountInfo.CalculateActualDiscountAmount(3);

            order.CalculatePaymentDiscount();

            Assert.Equal(98, order.PayAmount);
        }
    }
}