using System.Collections.Generic;
using PXPlus.Enum.Authorization;
using PXPlus.Payment.Domain.OrderAggregate;
using Xunit;

namespace PXPlus.Payment.Domain.Test.OrderAggregateTest
{
    public class OrderRefundDiscountPointTest
    {
        private Order NewOrder(decimal orderAmount, decimal noneDiscountAmount, decimal noneFeedbackAmount,
            int pointDiscountPercent)
            => new Order(new OrderNo("a","b"),null, null, orderAmount, noneDiscountAmount,
                noneFeedbackAmount, new PointDiscountInfo(30,false), PlatformType.PXPlus, new List<ActivityCashDiscountInfo>());


        //case 1.
        //退款 無點數折抵 無現金回饋 無點數回饋
        //付款 100 退款 80 -> OrderAmount = 20 , PayAmount = 20
        //退款 20 -> OrderAmount = 0 , PayAmount = 0
        [Fact]
        public void CaseOne()
        {
            //arrange pay 100 not point not any feedback
            var order = NewOrder(100, 0, 0, 30);

            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(0);
            order.CalculatePaymentDiscount();

            //action
            //refund 80 
            order.CalculateRefundDiscount(80);

            //assert
            // 100 - 80 = 20
            Assert.Equal(20, order.OrderAmount);
            // 10 - 80 = 20
            Assert.Equal(20, order.PayAmount);
        }

        //case 2.
        //付款101 上限30% 折抵28
        //退款29
        [Fact]
        public void CaseTwo()
        {
            //arrange
            //OrderAmount=101 DiscountPoint=28 可折抵金額 = [101*0.3] = 30
            var order = NewOrder(101, 0, 0, 30);
            var discountPoint = 28;
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(discountPoint);
            order.CalculatePaymentDiscount();

            //action
            //Refund 29
            order.CalculateRefundDiscount(29);

            //assert
            // 101 - 29 = 72
            Assert.Equal(72, order.OrderAmount);
            // 72 * 0.3 = [21.6] = 22
            Assert.Equal(22, order.PointDiscountInfo.AvailableDiscountAmount);
            // discountPoint > 22 => 22
            Assert.Equal(22, order.PointDiscountInfo.ActualDiscountAmount);
            // 72 -22 = 50
            Assert.Equal(50, order.PayAmount);
        }

        //case 3
        //付款101 上限30% 折抵28
        //退款29 
        //退款27
        [Fact]
        public void CaseThree()
        {
            //arrange
            //OrderAmount=101 DiscountPoint=28 可折抵金額 = [101*0.3]=30

            var discountPoint = 28;
            var order = NewOrder(101, 0, 0, 30);

            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(discountPoint);
            order.CalculatePaymentDiscount();
            //Refund 29 
            order.CalculateRefundDiscount(29);

            //action
            //Refund 29
            order.CalculateRefundDiscount(29);

            //assert
            // 72 - 29 = 43
            Assert.Equal(43, order.OrderAmount);
            // 43 * 0.3 = [12.9] = 13
            Assert.Equal(13, order.PointDiscountInfo.AvailableDiscountAmount);
            // discountPoint > 13 => 13
            Assert.Equal(13, order.PointDiscountInfo.ActualDiscountAmount);
            // 43 - 13 = 30
            Assert.Equal(30, order.PayAmount);
        }

        //Case 4
        //付款101 上限30% 折抵2元
        //退款29 
        [Fact]
        public void CaseFour()
        {
            //arrange
            //OrderAmount=101 DiscountPoint=2 可折抵金額 = [101*0.3] = 30
            var order = NewOrder(101, 0, 0, 30);
            var discountPoint = 2;
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(discountPoint);
            order.CalculatePaymentDiscount();

            //action
            //Refund 29
            order.CalculateRefundDiscount(29);

            //assert 
            // 101 - 29 = 72 
            Assert.Equal(72, order.OrderAmount);
            // 72 * 0.3 = [21.6] = 22
            Assert.Equal(22, order.PointDiscountInfo.AvailableDiscountAmount);
            //discountPoint < 22 => 2
            Assert.Equal(2, order.PointDiscountInfo.ActualDiscountAmount);
            // 72 - 2 =70
            Assert.Equal(70, order.PayAmount);
        }

        //Case 5 
        //付款101 上限30% 折抵2元
        //退款29 
        //退款29
        [Fact]
        public void CaseFive()
        {
            //arrange
            //OrderAmount=101 DiscountPoint=2 可折抵金額 = [101*0.3] = 30
            var order = NewOrder(101, 0, 0, 30);
            var discountPoint = 2;
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(discountPoint);
            order.CalculatePaymentDiscount();
            //Refund29
            order.CalculateRefundDiscount(29);

            //action
            //Refund 29
            order.CalculateRefundDiscount(29);

            //assert
            // 72 - 29 = 43
            Assert.Equal(43, order.OrderAmount);
            // 43 * 0.3 = [12.9] = 13
            Assert.Equal(13, order.PointDiscountInfo.AvailableDiscountAmount);
            // discountPoint < 13 => 2
            Assert.Equal(2, order.PointDiscountInfo.ActualDiscountAmount);
            // 43 - 2 = 41
            Assert.Equal(41, order.PayAmount);
        }

        //Case 6
        //付款101 上限30% 折抵2元
        //退款29 
        //退款29
        //退款43
        [Fact]
        public void CaseSix()
        {
            //arrange
            //OrderAmount=101 DiscountPoint=2 可折抵金額 = [101*0.3] = 30
            var order = NewOrder(101, 0, 0, 30);
            var discountPoint = 2;
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(discountPoint);
            order.CalculatePaymentDiscount();
            //Refund29
            order.CalculateRefundDiscount(29);
            //Refund 29
            order.CalculateRefundDiscount(29);

            //action
            //Refund 43
            order.CalculateRefundDiscount(43);

            //assert
            // 43 - 43 = 0
            Assert.Equal(0, order.OrderAmount);
            // 0 * 0.3 = [0] = 0
            Assert.Equal(0, order.PointDiscountInfo.AvailableDiscountAmount);
            // discountPoint < 0 =>0 
            Assert.Equal(0, order.PointDiscountInfo.ActualDiscountAmount);
            // 0 - 0 = 0
            Assert.Equal(0, order.PayAmount);
        }
    }
}