﻿using System.Collections.Generic;
using PXPlus.Enum.Authorization;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderAggregate.PayTools;
using Xunit;

namespace PXPlus.Payment.Domain.Test.OrderAggregateTest
{
    public class OrderRefundNoneDiscountTest
    {
        private Order NewOrder(decimal orderAmount, decimal noneDiscountAmount, decimal noneFeedbackAmount)
            => new Order(new OrderNo("a","b"),new PayerInfo(1,"a",PayToolType.EPAccount,0,new EpAccountPayTool("a"),"ph"),new ReceiveStoreInfo(
                    2,"mer","name","st","asd",1,2,1,2,1,"brand_name"), orderAmount, noneDiscountAmount,
                noneFeedbackAmount, new PointDiscountInfo(30,false), PlatformType.PXPlus, new List<ActivityCashDiscountInfo>());

        /// <summary>
        /// case1
        /// step 1.付款 1000 , 不可折抵 500 , 扣150點 , PayAmount = 850 , DiscountPoint = 150
        /// step 2.退款 500 不可折抵 0
        /// result OrderAmount = 500 , PayAmount = 500 , DiscountPoint = 0
        /// return PayAmount = 350 , DiscountPoint = 150 
        /// </summary>
        [Fact]
        public void Case1()
        {

            var order = NewOrder(1000, 500, 0);
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(150);
            order.CalculatePaymentDiscount();
            order.PaymentComplete(true);
            Assert.Equal(1000,order.OrderAmount);
            Assert.Equal(850,order.PayAmount);
            Assert.Equal(150,order.PointDiscountInfo.ActualDiscountAmount);
            
            order.TransformRefundOrder(new OrderNo("a","v"),order.PayerInfo.PayerId,order.ReceiveStoreInfo.StoreCode,500,0,0,"",0,"");
            
            
            order.CalculateRefundDiscount(500);
            
            Assert.Equal(500,order.OrderAmount);
            Assert.Equal(500,order.PayAmount);
            Assert.Equal(0,order.PointDiscountInfo.ActualDiscountAmount);
        }
        
        /// <summary>
        /// case 2
        /// step 1.付款 1000 , 不可折抵 500 , 扣100點 , PayAmount = 900 , DiscountPoint = 100
        /// step 2.退款 500 不可折抵 0
        /// result OrderAmount = 500 ,PayAmount = 500 , DiscountPoint = 0
        /// return PayAmount = 400 , DiscountPoint = 100 
        /// </summary>
        [Fact]
        public void Case2()
        {
            var order = NewOrder(1000, 500, 0);
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(100);
            order.CalculatePaymentDiscount();
            order.PaymentComplete(true);
            
            Assert.Equal(1000,order.OrderAmount);
            Assert.Equal(900,order.PayAmount);
            Assert.Equal(100,order.PointDiscountInfo.ActualDiscountAmount);
            
            order.TransformRefundOrder(new OrderNo("a","c"),order.PayerInfo.PayerId,order.ReceiveStoreInfo.StoreCode,500,0,0,"",0,"");
            
            
            
            order.CalculateRefundDiscount(500);
            
            Assert.Equal(500,order.OrderAmount);
            Assert.Equal(500,order.PayAmount);
            Assert.Equal(0,order.PointDiscountInfo.ActualDiscountAmount);
        }
        
        
        /// <summary>
        /// case 3
        /// step 1.付款 1000 , 不可折抵 500 , 扣150點 ,PayAmount = 850 , DiscountPoint = 150
        /// step 2.退款 100 不可折抵 0
        /// result OrderAmount = 900 , PayAmount = 780 , DiscountPoint = 120
        /// return PayAmount = 70 , point = 30
        /// </summary>
        [Fact]
        public void Case3()
        {
            var order = NewOrder(1000, 500, 0);
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(150);
            order.CalculatePaymentDiscount();
            order.PaymentComplete(true);
            
            Assert.Equal(1000,order.OrderAmount);
            Assert.Equal(850,order.PayAmount);
            Assert.Equal(150,order.PointDiscountInfo.ActualDiscountAmount);
            
            order.TransformRefundOrder(new OrderNo("a","v"),order.PayerInfo.PayerId,order.ReceiveStoreInfo.StoreCode,100,0,0,"",0,"");
            
            
            order.CalculateRefundDiscount(100);
            
            Assert.Equal(900,order.OrderAmount);
            Assert.Equal(780,order.PayAmount);
            Assert.Equal(120,order.PointDiscountInfo.ActualDiscountAmount);
        }
        
        /// <summary>
        /// case 4
        /// step 1.付款 1000 , 不可折抵 500 , 扣150點 ,PayAmount = 850 , DiscountPoint = 150
        /// step 2.退款 400 不可折抵 300
        /// result OrderAmount = 600 , PayAmount = 480 , DiscountPoint = 120
        /// return OrderAmount = 400 , PayAmount = 370 , point = 30
        /// </summary>
        [Fact]
        public void Case4()
        {
            var order = NewOrder(1000, 500, 0);
            order.CalculateAvailablePointDiscount();
            order.PointDiscountInfo.CalculateActualDiscountAmount(150);
            order.CalculatePaymentDiscount();
            order.PaymentComplete(true);
            
            Assert.Equal(1000,order.OrderAmount);
            Assert.Equal(850,order.PayAmount);
            Assert.Equal(150,order.PointDiscountInfo.ActualDiscountAmount);
            
            order.TransformRefundOrder(new OrderNo("a","c"),order.PayerInfo.PayerId,order.ReceiveStoreInfo.StoreCode,400,300,0,"",0,"");
            
            
            order.CalculateRefundDiscount(400);
            
            Assert.Equal(600,order.OrderAmount);
            Assert.Equal(480,order.PayAmount);
            Assert.Equal(120,order.PointDiscountInfo.ActualDiscountAmount);
        }
        
    }
}