﻿using PXPlus.Core.Exception;

namespace PXPlus.Payment.Domain.DomainExceptions
{
    public class OrderDomainException : PXPlusException
    {
        private const string PaymentPrefix = "PT";
        public string AdditionalMessage { get; }
        public OrderDomainException(string resultCodeStr, string resultMessage, string prefix) : base(
            resultCodeStr, resultMessage, prefix)
        {
        }

        public OrderDomainException(string resultCodeStr, string resultMessage) : base(
            resultCodeStr, resultMessage, PaymentPrefix)
        {
        }

        public OrderDomainException(PaymentStatusCode @enum,string additionalMessage = "") : base(@enum, PaymentPrefix)
        {
            AdditionalMessage = additionalMessage;
        }

        public OrderDomainException(PaymentStatusCode @enum, string exceptionMessage,string additionalMessage ="") : base(@enum,
            exceptionMessage, PaymentPrefix)
        {
            AdditionalMessage = additionalMessage;
        }
    }

    public enum PaymentStatusCode
    {
        /// <summary>
        /// 交易成功
        /// </summary>
        [PXPlusResultCodeAttribute("0000")]
        [PXPlusResultMessageAttribute("交易成功")]
        TradeSuccess = 0000,

        /// <summary>
        /// 訂單處理中 
        /// </summary>
        [PXPlusResultCodeAttribute("0001")]
        [PXPlusResultMessageAttribute("訂單處理中")]
        TradeProcessing = 0001,
        
        /// <summary>
        /// 全支付帳戶餘額不足
        /// </summary>
        [PXPlusResultCodeAttribute("0002")]
        [PXPlusResultMessageAttribute("全支付帳戶餘額不足")]
        EpAccountPayFailed = 0002,
        
        /// <summary>
        /// 信用卡授權失敗
        /// </summary>
        [PXPlusResultCodeAttribute("0003")]
        [PXPlusResultMessageAttribute("信用卡授權失敗")]
        CreditCardAuthFailed = 0003,

        /// <summary>
        /// 銀行帳戶扣款失敗
        /// </summary>
        [PXPlusResultCodeAttribute("0004")]
        [PXPlusResultMessageAttribute("銀行帳戶扣款失敗")]
        AccountLinkPayFailed = 0004,

        /// <summary>
        /// 退款失敗(未找到原單)
        /// </summary>
        [PXPlusResultCodeAttribute("0005")]
        [PXPlusResultMessageAttribute("退款失敗(未找到原單)")]
        RefundFailedOrderNotFound = 0005,

        /// <summary>
        /// 退款失敗(可用餘額不足)
        /// </summary>
        [PXPlusResultCodeAttribute("0006")]
        [PXPlusResultMessageAttribute("退款失敗(可用餘額不足)")]
        RefundFailedAvailableAmountNotEnough = 0006,

        /// <summary>
        /// 店家不存在
        /// </summary>
        [PXPlusResultCodeAttribute("0007")]
        [PXPlusResultMessageAttribute("店家不存在")]
        StoreIsNotExist = 0007,

        /// <summary>
        /// 信用卡有效期已過期
        /// </summary>
        [PXPlusResultCodeAttribute("0008")]
        [PXPlusResultMessageAttribute("信用卡有效期已過期")]
        CreditCardIsExpired = 0008,

        /// <summary>
        /// 退款失敗(原因)
        /// </summary>
        [PXPlusResultCodeAttribute("0009")]
        [PXPlusResultMessageAttribute("退款失敗")]
        RefundFailed = 0009,
        
        /// <summary>
        /// 參數無效
        /// </summary>
        [PXPlusResultCodeAttribute("0010")]
        [PXPlusResultMessageAttribute("參數無效")]
        InvalidParameter = 0010,
        
        /// <summary>
        /// 查無會員
        /// </summary>
        [PXPlusResultCodeAttribute("0011")]
        [PXPlusResultMessageAttribute("查無會員")]
        MemberNotFound = 0011,
        
        /// <summary>
        /// 訂單待確認
        /// </summary>
        [PXPlusResultCodeAttribute("0012")]
        [PXPlusResultMessageAttribute("訂單待確認")]
        OrderToBeConfirm = 0012,
        
        /// <summary>
        /// 訂單確認失敗
        /// </summary>
        [PXPlusResultCodeAttribute("0013")]
        [PXPlusResultMessageAttribute("訂單確認失敗")]
        OrderConfirmFailed = 0013,

        
        /// <summary>
        /// TradeNo已被使用
        /// </summary>
        [PXPlusResultCodeAttribute("0014")]
        [PXPlusResultMessageAttribute("TradeNo已被使用")]
        TradeNoHasBeenUsed = 0014,
        
        /// <summary>
        /// 稍後再試
        /// </summary>
        [PXPlusResultCodeAttribute("9998")]
        [PXPlusResultMessageAttribute("稍後再試")]
        TryAgainLater = 9998,

        /// <summary>
        /// 其他錯誤
        /// </summary>
        [PXPlusResultCodeAttribute("9999")]
        [PXPlusResultMessageAttribute("其他錯誤")]
        OtherError = 9999
    }
}