using MediatR;
using PXPlus.Payment.Domain.OrderAggregate;

namespace PXPlus.Payment.Domain.Events
{
    public class OrderRefundSuccessfulDomainEvent : INotification
    {
        public Order Order { get; set; }
        /// <summary>
        /// 退款金額
        /// </summary>
        public decimal RefundOrderAmount { get; set; }
        /// <summary>
        /// 退還消費者的錢
        /// </summary>
        public decimal RefundPayAmount { get; set; }
        /// <summary>
        /// 退還消費者的點數
        /// </summary>
        public decimal RefundPointAmount { get; set; }

        /// <summary>
        /// 不可折抵金額
        /// </summary>
        public decimal NoneDiscountAmount { get; set; }
        
        /// <summary>
        /// 不可回饋金額
        /// </summary>
        public decimal NoneFeedbackAmount { get; set; }

        /// <summary>
        /// 扣回現金回饋
        /// </summary>
        public decimal CashBack { get; set; }
        
        /// <summary>
        /// 扣回點數回饋
        /// </summary>
        public decimal PointBack { get; set; }

        public OrderRefundSuccessfulDomainEvent(Order order, decimal refundOrderAmount, decimal refundPayAmount,
            decimal refundPointAmount,decimal noneDiscountAmount,decimal noneFeedbackAmount,decimal cashBack,decimal pointBack)
        {
            Order = order;
            RefundOrderAmount = refundOrderAmount;
            RefundPayAmount = refundPayAmount;
            RefundPointAmount = refundPointAmount;
            NoneDiscountAmount = noneDiscountAmount;
            NoneFeedbackAmount = noneFeedbackAmount;
            CashBack = cashBack;
            PointBack = pointBack;
        }
    }
}