﻿using System.Collections.Generic;
using PXPlus.Enum.Authorization;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.Factories.Interface;
using PXPlus.Payment.Domain.OrderAggregate;

namespace PXPlus.Payment.Domain.Factories.Implement
{
    public class OrderFactory : IOrderFactory
    {
        private readonly IGenerateOrderNoDomainService _generateOrderNoDomainService;
        public OrderFactory(IGenerateOrderNoDomainService generateOrderNoDomainService)
        {
            _generateOrderNoDomainService = generateOrderNoDomainService;
        }
        
        public Order CreateInitialOrder(PayerInfo payerInfo, bool isUsePoint,
            decimal orderAmount, decimal pointDiscountPercent, decimal noneDiscountAmount,
            decimal noneFeedbackAmount,
            ReceiveStoreInfo receiveStoreInfo,
            PlatformType platformType, List<ActivityCashDiscountInfo> activityCashDiscountInfos,
            MetaData inputMetaData)
        {
           
            var pointDiscountInfo = new PointDiscountInfo(pointDiscountPercent,isUsePoint);

            var orderNo = _generateOrderNoDomainService.GenerateOrderNo(receiveStoreInfo.MerchantCode);
            
            var order = new Order(orderNo,payerInfo, receiveStoreInfo, orderAmount, noneDiscountAmount, noneFeedbackAmount,
                pointDiscountInfo, platformType, activityCashDiscountInfos);
            
            order.SetInputMetaData(inputMetaData);
            return order;
        }
    }
}