﻿using System;
using PXPlus.Payment.Domain.OrderAggregate.PayTools;

namespace PXPlus.Payment.Domain.OrderAggregate
{
    public record PayerInfo
    {
        public long PayerId { get; }
        public PayToolType PayToolType { get; }
        public string PayerPaymentAccount { get; private set; }
        public int Sequence { get; }
        
        public string PayerEpAccount{ get; }
        private  CreditCardPayTool CreditCardPayTool { get; set; }
        private  AccountLinkPayTool AccountLinkPayTool{ get; set; }
        private  EpAccountPayTool EpAccountPayTool{ get; set; }

        public string PhoneNumber { get; }

        private string _bankCode;
        private string _bankName;
        
        //for ef core
        private PayerInfo()
        {
            
        }
        
        public PayerInfo(long payerId, string payerEpAccount, PayToolType payToolType, int sequence,
            IPayTool payTool,string phoneNumber)
        {
            PayerId = payerId;
            PayerEpAccount = payerEpAccount;
            PhoneNumber = phoneNumber;
            PayToolType = payToolType;
            Sequence = sequence;
            SetPayTool(payTool);
        }

        public void SetPayTool(IPayTool payTool)
        {
            switch (PayToolType)
            {
                case PayToolType.AccountLink:
                    AccountLinkPayTool = (AccountLinkPayTool) payTool;
                    PayerPaymentAccount = AccountLinkPayTool.VirtualAccount;
                    _bankCode = AccountLinkPayTool.BankCode;
                    _bankName = AccountLinkPayTool.BankName;
                    break;
                case PayToolType.CreditCard:
                    CreditCardPayTool = (CreditCardPayTool) payTool;
                    PayerPaymentAccount = CreditCardPayTool.CreditCard;
                    _bankCode = CreditCardPayTool.BankCode;
                    _bankName = CreditCardPayTool.BankName;
                    break;
                case PayToolType.EPAccount:
                    EpAccountPayTool = (EpAccountPayTool) payTool;
                    PayerPaymentAccount = EpAccountPayTool.EpAccount;
                    break;
                case PayToolType.Unknown:
                    break;
                default:
                    throw new ArgumentOutOfRangeException(nameof(PayToolType), PayToolType, null);
            }
        }

        public string GetMaskedPayerPaymentAccount()
        {
            return PayToolType switch
            {
                PayToolType.AccountLink => AccountLinkPayTool.GetMaskedPayToolAccount(),
                PayToolType.CreditCard => CreditCardPayTool.GetMaskedPayToolAccount(),
                PayToolType.EPAccount => EpAccountPayTool.GetMaskedPayToolAccount(),
                _ => throw new ArgumentOutOfRangeException()
            };
        }

        public IPayTool GetPayTool()
            => PayToolType switch
            {
                PayToolType.AccountLink => AccountLinkPayTool,
                PayToolType.CreditCard => CreditCardPayTool,
                PayToolType.EPAccount => EpAccountPayTool,
                _ => throw new ArgumentOutOfRangeException()
            };

        public string GetBankCode()
            => _bankCode;

        public string GetBankName()
            => _bankName;
    }
}