﻿using System;

namespace PXPlus.Payment.Domain.OrderAggregate
{
    public class PointDiscountInfo
    {
        public bool IsUsePoint { get; private set; }
        public decimal DiscountPercent { get;private  set; }
        public decimal AvailableDiscountAmount { get;private  set; }
        public decimal ActualDiscountAmount { get;private  set; }

        public PointDiscountInfo(decimal discountPercent, bool isUsePoint)
        {
            if (discountPercent < 0 || discountPercent > 100)
                throw new ArgumentOutOfRangeException(nameof(discountPercent)+" between 0 ~ 100");

            DiscountPercent = discountPercent;
            IsUsePoint = isUsePoint;
        }

        public void CalculateAvailableDiscount(decimal availableDiscountAmount)
        {
            if (availableDiscountAmount < 0)
                throw new ArgumentOutOfRangeException(nameof(availableDiscountAmount) + " must be positive.");

            AvailableDiscountAmount = Math.Round(availableDiscountAmount * DiscountPercent / 100,0,MidpointRounding.AwayFromZero);
        }

        public void CalculateActualDiscountAmount(decimal discountAmount)
        {
            if (discountAmount < 0)
                throw new ArgumentOutOfRangeException(nameof(discountAmount) + " must be positive.");
            
            if (AvailableDiscountAmount > discountAmount)
                ActualDiscountAmount = discountAmount;
            else
                ActualDiscountAmount = AvailableDiscountAmount;
        }
    }
}