﻿using System.Collections.Generic;

namespace PXPlus.Payment.Domain.OrderAggregate
{
    /// <summary>
    /// 收款店家
    /// </summary>
    public class ReceiveStoreInfo
    {
        /// <summary>
        /// 商家ID
        /// </summary>
        public string MerchantCode { get; }

        public string MerchantName { get; }
        
        /// <summary>
        /// 店家ID
        /// </summary>
        public string StoreCode { get; }
    
        /// <summary>
        /// 店家名稱
        /// </summary>
        public string StoreName { get; }
        
        /// <summary>
        /// 店家的會員ID
        /// </summary>
        public long StoreMemberId { get; }
        /// <summary>
        /// 收款Ep帳號
        /// </summary>
        public string ReceiveAccount { get; private set; }
        /// <summary>
        /// 交易手續費費率
        /// </summary>
        public decimal TradeFeeAmount { get; }
        /// <summary>
        /// 交易手續費算法
        /// </summary>
        public int TradeFeeUnit { get; }
        /// <summary>
        /// 點數折抵手續費費率
        /// </summary>
        public decimal PointDiscountFeeRate { get; }
        /// <summary>
        /// 點數折抵上限
        /// </summary>
        public decimal PointDiscountPercent { get; }
        public int IndustryType { get; }

        public string BrandName { get; }

        public string MerchantTradeNo { get; private set; }
        //todo
        public IEnumerable<Fee> Fees { get; set; }

        public ReceiveStoreInfo(long storeMemberId,string merchantCode,string merchantName,string storeCode,string storeName
        ,int industryType,decimal pointDiscountFeeRate,int tradeFeeUnit,decimal tradeFeeAmount,decimal pointDiscountPercent,
        string brandName)
        {
            StoreMemberId = storeMemberId;
            MerchantCode = merchantCode;
            MerchantName = merchantName;
            StoreCode = storeCode;
            StoreName = storeName;
            IndustryType = industryType;
            PointDiscountFeeRate = pointDiscountFeeRate;
            TradeFeeUnit = tradeFeeUnit;
            TradeFeeAmount = tradeFeeAmount;
            PointDiscountPercent = pointDiscountPercent;
            BrandName = brandName;
        }

        public void SetReceiveAccount(string receiveAccount)
        {
            ReceiveAccount = receiveAccount;
        }

        public void SetMerchantTradeNo(string merchantTradeNo)
        {
            MerchantTradeNo = merchantTradeNo;
        }

        // for ef core
        private ReceiveStoreInfo()
        {
            
        }
    }
}