﻿using System;
using System.Collections.Generic;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.SeedWork;

namespace PXPlus.Payment.Domain.OrderRecordAggregate
{
    public partial class OrderRecord : Entity, IAggregateRoot
    {
        /// <summary>
        /// 原單Id
        /// </summary>
        public long OrderId { get; private set; }
        
        /// <summary>
        /// 訂單號
        /// </summary>
        public string OrderNo { get; private set; }

        /// <summary>
        /// 紀錄類別
        /// </summary>
        public OrderRecordType OrderRecordType { get; private set; }

        /// <summary>
        /// 紀錄結果
        /// </summary>
        public OrderRecordResult OrderRecordResult { get; private set; }

        /// <summary>
        /// 紀錄時間
        /// </summary>
        public DateTime RecordTime { get; private set; }

        /// <summary>
        /// 訂單時間
        /// </summary>
        public DateTime OrderTime { get; private set; }

        /// <summary>
        /// 訂單金額
        /// </summary>
        public decimal OrderAmount { get; private set; }

        /// <summary>
        /// 支付金額
        /// </summary>
        public decimal PayAmount { get; private set; }

        /// <summary>
        /// 點數折抵
        /// </summary>
        public decimal ActualDiscountAmount { get; private set; }

        /// <summary>
        /// 不可折抵金額
        /// </summary>
        public decimal NoneDiscountAmount { get; private set; }
        
        
        /// <summary>
        /// 不可回饋金額
        /// </summary>
        public decimal NoneFeedbackAmount { get; private set; }

        /// <summary>
        /// 活動現金折抵
        /// </summary>
        public IReadOnlyCollection<ActivityCashDiscountInfo> ActivityCashDiscountInfos { get; private set; }

        /// <summary>
        /// 活動回饋
        /// </summary>
        public IReadOnlyCollection<ActivityFeedbackInfo> ActivityFeedbackInfos { get; private set; }

        /// <summary>
        /// 點數回饋
        /// </summary>
        public decimal CashBack { get; set; }

        /// <summary>
        /// 現金回饋
        /// </summary>
        public decimal PointBack { get; set; }

        public void SetRecordTypeAndResult(OrderRecordResult orderRecordResult)
        {
            OrderRecordResult = orderRecordResult;
        }
    }

    public partial class OrderRecord
    {
        protected OrderRecord(){}
        // payment order
        public static OrderRecord FromOrderDomainModelForPaymentOrder(Order order)
            => new OrderRecord
               {
                   OrderId = order.Id,
                   OrderNo = order.OrderNo.GetOrderNo(),
                   RecordTime = DateTime.Now,
                   OrderTime = order.GetUpdatedTime,
                   OrderAmount = order.OrderAmount,
                   PayAmount = order.PayAmount,
                   ActualDiscountAmount = order.PointDiscountInfo.ActualDiscountAmount,
                   NoneDiscountAmount = order.NoneDiscountAmount,
                   NoneFeedbackAmount = order.NoneFeedbackAmount,
                   ActivityCashDiscountInfos = order.ActivityCashDiscountInfos,
                   ActivityFeedbackInfos = order.ActivityFeedbackInfos,
                   OrderRecordType = OrderRecordType.Payment,
                   CashBack = order.BackInfo.CashBack,
                   PointBack = order.BackInfo.PointBack
               };
        // refund order
        public static OrderRecord FromOrderDomainModelForRefundOrder(Order order,string refundOrderNo,decimal refundOrderAmount
        ,decimal refundPayAmount,decimal refundPoint,decimal noneDiscountAmount,decimal noneFeedbackAmount)
            => new OrderRecord
               {
                   OrderId = order.Id,
                   OrderNo = refundOrderNo,
                   RecordTime = DateTime.Now,
                   OrderTime = order.GetUpdatedTime,
                   OrderAmount = refundOrderAmount,
                   PayAmount = refundPayAmount,
                   ActualDiscountAmount = refundPoint,
                   NoneDiscountAmount = noneDiscountAmount,
                   NoneFeedbackAmount = noneFeedbackAmount,
                   OrderRecordType = OrderRecordType.Refund
               };
    }
}