using System;
using System.Threading.Tasks;
using CreditCardPaymentService;
using Microsoft.Extensions.Logging;
using PXPlus.Core.Extensions;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderAggregate.PayTools;

namespace PXPlus.Payment.Infrastructure.DomainServiceGrpc.ExecutePaymentDomainServices
{
    public class ExecuteCreditCardPaymentGrpcDomainService : IExecutePaymentDomainService
    {
        private readonly PayGrpcService.PayGrpcServiceClient _payGrpcServiceClient;
        private readonly ILogger<ExecuteCreditCardPaymentGrpcDomainService> _logger;

        public ExecuteCreditCardPaymentGrpcDomainService(PayGrpcService.PayGrpcServiceClient payGrpcServiceClient, ILogger<ExecuteCreditCardPaymentGrpcDomainService> logger)
        {
            _payGrpcServiceClient = payGrpcServiceClient;
            _logger = logger;
        }
        
        public async Task<ExecutePaymentInfo> PaymentAsync(string pxTradeNo, decimal payAmount, IPayTool payTool,
            ReceiveStoreInfo storeInfo)
        {
            try
            {
                if (! (payTool is CreditCardPayTool creditCardPayTool))
                    throw new ArgumentNullException(nameof(CreditCardPayTool));

                var payGrpcRequest = new PayGrpcRequest
                                     {
                                         TradeNo = pxTradeNo,
                                         CreditCard = creditCardPayTool.CreditCard,
                                         ExpiryDate = creditCardPayTool.ExpiryDate,
                                         Amount = (double) payAmount,
                                         MerchantCode = storeInfo.MerchantCode
                                     };
                
                var response = await _payGrpcServiceClient.PayAsync(payGrpcRequest);

                var creditcardPaymentInfo = new ExecutePaymentInfo
                {
                    IsSuccess = false,
                    Message = response.Message,
                    Code = PaymentStatusCode.CreditCardAuthFailed.GetResultCode(),
                    BankReturnCode = "todo"
                };

                if (response.Code != "0000")
                    return creditcardPaymentInfo;
                
                creditcardPaymentInfo.IsSuccess = true;
                creditcardPaymentInfo.Code = response.Code;

                return creditcardPaymentInfo;
            }
            catch (Exception e)
            {
                _logger.LogError(e,e.Message);
                return new ExecutePaymentInfo
                       {
                           IsSuccess = false,
                           Message = e.Message,
                           Code = PaymentStatusCode.CreditCardAuthFailed.GetResultCode()
                       };
            }
            
        }

        public Task<ExecuteRefundInfo> RefundAsync(string pxOriTradeNo, string pxRefundTradeNo, decimal payAmount,
            PayerInfo payerInfo, ReceiveStoreInfo storeInfo)
        {
            throw new NotImplementedException();
        }
    }
}