using System;
using System.Threading.Tasks;
using AccountForPaymentService;
using Microsoft.Extensions.Logging;
using PXPlus.Core.Extensions;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderAggregate.PayTools;

namespace PXPlus.Payment.Infrastructure.DomainServiceGrpc.ExecutePaymentDomainServices
{
    public class ExecuteEpAccountPaymentGrpcDomainService : IExecutePaymentDomainService
    {
        private readonly EpAccountForPaymentGrpcService.EpAccountForPaymentGrpcServiceClient _epAccountForPaymentGrpcServiceClient;
        private readonly ILogger<ExecuteEpAccountPaymentGrpcDomainService> _logger;

        public ExecuteEpAccountPaymentGrpcDomainService(EpAccountForPaymentGrpcService.EpAccountForPaymentGrpcServiceClient epAccountForPaymentGrpcServiceClient, ILogger<ExecuteEpAccountPaymentGrpcDomainService> logger)
        {
            _epAccountForPaymentGrpcServiceClient = epAccountForPaymentGrpcServiceClient;
            _logger = logger;
        }
        public async Task<ExecutePaymentInfo> PaymentAsync(string pxTradeNo, decimal payAmount,
            IPayTool payTool,
            ReceiveStoreInfo storeInfo)
        {
            try
            {
                if (! (payTool is EpAccountPayTool epAccountPayTool))
                    throw new ArgumentNullException(nameof(EpAccountPayTool));
                
                var request = new EpAccountPayGrpcRequest
                            {
                                PayerEpAccount = epAccountPayTool.EpAccount,
                                ReceiverEpAccount = storeInfo.ReceiveAccount,
                                Amount = (double)payAmount,
                                TradeNo = pxTradeNo
                            };

                var response = await _epAccountForPaymentGrpcServiceClient.PayAsync(request);
                
                var accountPaymentInfo = new ExecutePaymentInfo
                {
                    IsSuccess = false,
                    Message = response.Message,
                    Code = PaymentStatusCode.EpAccountPayFailed.GetResultCode(),
                    BankReturnCode = response.Code
                };

                if (response.Code == "0000")
                {
                    accountPaymentInfo.IsSuccess = true;
                    accountPaymentInfo.Code = PaymentStatusCode.TradeSuccess.GetResultCode();
                }

                return accountPaymentInfo;
            }
            catch (Exception e)
            {
                _logger.LogError(e,e.Message);
                return new ExecutePaymentInfo
                       {
                            IsSuccess = false,
                            Message = e.Message,
                            Code = PaymentStatusCode.EpAccountPayFailed.GetResultCode()
                       };
            }

        }

        public async Task<ExecuteRefundInfo> RefundAsync(string pxOriTradeNo, string pxRefundTradeNo, decimal payAmount,
            PayerInfo payerInfo, ReceiveStoreInfo storeInfo)
        {
            try
            {
                var request = new EpAccountRefundGrpcRequest
                {
                    Amount = (double)payAmount,
                    TradeNo = pxOriTradeNo,
                    RefundTradeNo =pxRefundTradeNo 
                };

                var response = await _epAccountForPaymentGrpcServiceClient.RefundAsync(request);
                
                var accountPaymentInfo = new ExecuteRefundInfo
                {
                    IsSuccess = false,
                    Message = response.Message,
                    Code = PaymentStatusCode.RefundFailed.GetResultCode(),
                    BankReturnCode = response.Code
                };

                if (response.Code == "0000")
                {
                    accountPaymentInfo.IsSuccess = true;
                    accountPaymentInfo.Code = PaymentStatusCode.TradeSuccess.GetResultCode();
                }

                return accountPaymentInfo;
            }
            catch (Exception e)
            {
                _logger.LogError(e,e.Message);
                return new ExecuteRefundInfo
                {
                    IsSuccess = false,
                    Message = e.Message,
                    Code = PaymentStatusCode.RefundFailed.GetResultCode()
                };
            }
        }
    }
}