﻿using System;
using System.Threading.Tasks;
using MerchantForPaymentService;
using Microsoft.Extensions.Logging;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.OrderAggregate;

namespace PXPlus.Payment.Infrastructure.DomainServiceGrpc
{
    public class MerchantGrpcDomainService : IMerchantDomainService
    {
        private readonly MerchantForPaymentGrpcService.MerchantForPaymentGrpcServiceClient
            _merchantForPaymentGrpcServiceClient;

        private readonly ILogger<MerchantGrpcDomainService> _logger;

        public MerchantGrpcDomainService(MerchantForPaymentGrpcService.MerchantForPaymentGrpcServiceClient merchantForPaymentGrpcServiceClient, ILogger<MerchantGrpcDomainService> logger)
        {
            _merchantForPaymentGrpcServiceClient = merchantForPaymentGrpcServiceClient;
            _logger = logger;
        }
        public async Task<ReceiveStoreInfo> GetStoreAsync(string storeCode, int payToolType)
        {
            try
            {
                var response = await _merchantForPaymentGrpcServiceClient.GetStoreTransInfoAsync(
                    new GetStoreTransInfoGrpcRequest
                    {
                        PaytoolType = payToolType,
                        StoreCode = storeCode
                    });

                if (response.Code != "0000")
                    throw new OrderDomainException(PaymentStatusCode.StoreIsNotExist);

                return MapTo(storeCode,response);
            }
            catch (OrderDomainException)
            {
                throw;
            }
            catch (Exception e)
            {
                _logger.LogError(e,e.Message);
                throw new OrderDomainException(PaymentStatusCode.StoreIsNotExist);
            }
        }

        private ReceiveStoreInfo MapTo(string storeCode,GetStoreTransInfoGrpcResponse response)
        {
            var data = response.Data;
            return new ReceiveStoreInfo(
                data.MemberId
                ,data.MerchantCode
                ,data.MerchantName
                ,storeCode
                ,data.StoreName
                ,data.IndustryType
                ,(decimal)data.PointDiscountFeeRate
                ,data.TradeFeeUnit
                ,(decimal)data.TradeFeeAmount
                ,(decimal)data.PointDiscountPercent
                ,response.Data.BrandName);
        }
    }
}