using System;
using System.Threading.Tasks;
using AccountForPaymentService;
using Microsoft.Extensions.Logging;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.PaymentQuotaAggregate;

namespace PXPlus.Payment.Infrastructure.DomainServiceGrpc
{
    public class PaymentQuotaGrpcDomainService : IPaymentQuotaDomainService
    {
        private readonly AccountForPaymentGrpcService.AccountForPaymentGrpcServiceClient
            _accountForPaymentGrpcServiceClient;
        private readonly ILogger<PaymentQuotaGrpcDomainService> _logger;

        public PaymentQuotaGrpcDomainService(AccountForPaymentGrpcService.AccountForPaymentGrpcServiceClient accountForPaymentGrpcServiceClient, ILogger<PaymentQuotaGrpcDomainService> logger)
        {
            _accountForPaymentGrpcServiceClient = accountForPaymentGrpcServiceClient;
            _logger = logger;
        }
        
        public async Task<PaymentQuota> VerifyPaymentQuotaAndReturnQuota(long payerId, long receiverId, decimal orderAmount)
        {
            try
            {
                var response = await _accountForPaymentGrpcServiceClient.VerifyPaymentQuotaAsync(new VerifyPaymentQuotaGrpcRequest
                {
                    PayerMemberId = payerId,
                    ReceiverMemberId = receiverId,
                    Amount = (double)orderAmount,
                    Lock = false
                });

                if (response.Code != "0000")
                    throw new OrderDomainException(response.Code, response.Message);

                var data = response.Data;
                
                return new PaymentQuota((decimal)data.PayerDailyQuota,(decimal)data.PayerMonthlyQuota);
            }
            catch (Exception e)
            {
                _logger.LogError(e,e.Message);
                throw;
            }
        }
    }
}