using System;
using System.Threading.Tasks;
using Google.Protobuf.WellKnownTypes;
using Microsoft.Extensions.Logging;
using PointGrpcService;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;

namespace PXPlus.Payment.Infrastructure.DomainServiceGrpc
{
    public class PointGrpcDomainService : IPointDomainService
    {
        private readonly ILogger<PointGrpcDomainService> _logger;
        private readonly OperatePointGrpcService.OperatePointGrpcServiceClient _client;

        public PointGrpcDomainService(ILogger<PointGrpcDomainService> logger, OperatePointGrpcService.OperatePointGrpcServiceClient client)
        {
            _logger = logger;
            _client = client;
        }
        public async Task<decimal> DeductedToNoneAsync(long payerId, decimal point, string orderNo,
            DateTime tradeTime)
        {
            try
            {
                var response = await _client.DeductPointAsync(new DeductPointInfoGrpcRequest
                {
                    PxOrderNo = orderNo,
                    MemberId = payerId,
                    DeductPoints = (float)point,
                    TransactionTime = tradeTime.ToUniversalTime().ToTimestamp()
                });

                if (response.Code != "0000")
                    throw new OrderDomainException(response.Code,response.Message);

                return (decimal)response.Data.ActualDeductPoints;
            }
            catch (Exception e)
            {
                _logger.LogError(e,e.Message);
                return 0;
            }            
        }

        public Task ReturnPointAsync(decimal point)
        {
            _logger.LogInformation($"-------Return-------{point} Point");
            return Task.CompletedTask;
        }
    }
}