﻿using System.Linq;
using IdGen;
using Microsoft.Extensions.Options;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Infrastructure.Configs;

namespace PXPlus.Payment.Infrastructure.DomainServiceImp
{
    public class GenerateOrderNoDomainService : IGenerateOrderNoDomainService
    {
        private readonly IOptionsMonitor<OrderNoPrefixConfig> _optionsMonitor;
        private const string DefaultPrefix = "TX";
        private readonly IdGenerator _idGenerator;

        public GenerateOrderNoDomainService(IOptionsMonitor<OrderNoPrefixConfig> optionsMonitor,IdGenerator idGenerator)
        {
            _optionsMonitor = optionsMonitor;
            _idGenerator = idGenerator;
        }
        
        public OrderNo GenerateOrderNo(string merchantCode)
        {
            var prefix = GetOrderNoPrefix(merchantCode);
            var serialNumber = _idGenerator.CreateId().ToString();
            return new OrderNo(prefix, serialNumber);
        }

        public OrderNo GenerateOrderNoWithPrefix(string prefix)
        {
            var serialNumber = _idGenerator.CreateId().ToString();
            return new OrderNo(prefix, serialNumber);
        }


        private string GetOrderNoPrefix(string merchantCode)
        {
            if (_optionsMonitor.CurrentValue.OrderPrefixes == null)
                return DefaultPrefix;

            var prefix = _optionsMonitor.CurrentValue.OrderPrefixes.FirstOrDefault(c => c.MerchantCode.Equals(merchantCode));
            
            if (prefix == null)
                return DefaultPrefix;
            
            return prefix.Prefix;
        }
    }
}