using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.EntityFrameworkCore.Metadata.Builders;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderRecordAggregate;

namespace PXPlus.Payment.Infrastructure.EntityConfigurations
{
    public class OrderRecordEntityTypeConfiguration: IEntityTypeConfiguration<OrderRecord>
    {
        public void Configure(EntityTypeBuilder<OrderRecord> builder)
        {
            builder.ToTable("OrderRecords");
            builder.HasKey(c => c.Id);
            builder.HasIndex(c => c.OrderNo);
            builder.HasIndex(c => c.OrderId);

            builder.Property(c => c.OrderNo)
                   .HasColumnType("varchar(25)");

            builder.Property(c => c.OrderRecordResult)
                   .HasColumnType("smallint");

            builder.Property(c => c.OrderRecordType)
                   .HasColumnType("smallint");
            
            
            builder.Property(c=>c.ActivityFeedbackInfos) 
                     .HasColumnName("ActivityFeedbackInfos")
                     .HasColumnType("varchar(max)")
                     .HasConversion(
                         v => JsonSerializer.Serialize(v, null),
                         v => JsonSerializer.Deserialize<IReadOnlyCollection<ActivityFeedbackInfo>>(v, null),
                         new ValueComparer<IReadOnlyCollection<ActivityFeedbackInfo>>(
                             (c1, c2) => c1.SequenceEqual(c2),
                             c => c.Aggregate(0, (a, v) => HashCode.Combine(a, v.GetHashCode())),
                             c => c)
                     );

            
            builder.Property(c=>c.ActivityCashDiscountInfos)
                   .HasColumnName("ActivityCashDiscountInfos")
                   .HasColumnType("varchar(max)")
                   .HasConversion(
                       v => JsonSerializer.Serialize(v, null),
                       v => JsonSerializer.Deserialize<IReadOnlyCollection<ActivityCashDiscountInfo>>(v, null),
                       new ValueComparer<IReadOnlyCollection<ActivityCashDiscountInfo>>(
                           (c1, c2) => c1.SequenceEqual(c2),
                           c => c.Aggregate(0, (a, v) => HashCode.Combine(a, v.GetHashCode())),
                           c =>c)
                   );
        }
    }
}