using System;
using System.Collections.Generic;
using System.Linq;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.Factories.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Infrastructure.DomainServiceGrpc.ExecutePaymentDomainServices;

namespace PXPlus.Payment.Infrastructure.Factories
{
    public class ExecutePaymentDomainServiceFactory : IExecutePaymentDomainServiceFactory
    {
        private readonly IEnumerable<IExecutePaymentDomainService> _executePaymentDomainServices;

        public ExecutePaymentDomainServiceFactory(
            IEnumerable<IExecutePaymentDomainService> executePaymentDomainServices)
        {
            _executePaymentDomainServices = executePaymentDomainServices;
        }

        public IExecutePaymentDomainService Build(PayToolType payToolType)
        {
            switch (payToolType)
            {
                case PayToolType.AccountLink:
                    return _executePaymentDomainServices.FirstOrDefault(c =>
                        c.GetType() == typeof(ExecuteAcLinkPaymentGrpcDomainService));
                
                case PayToolType.CreditCard:
                    return _executePaymentDomainServices.FirstOrDefault(c =>
                        c.GetType() == typeof(ExecuteCreditCardPaymentGrpcDomainService));
                
                case PayToolType.EPAccount:
                    return _executePaymentDomainServices.FirstOrDefault(c =>
                        c.GetType() == typeof(ExecuteEpAccountPaymentGrpcDomainService));
                
                default:
                    throw new ArgumentOutOfRangeException(nameof(payToolType), payToolType, null);
            }
        }
    }
}