using System;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;

namespace PXPlus.Payment.Infrastructure.Idempotency
{
    public class RequestManager : IRequestManager
    {
        private readonly PaymentContext _paymentContext;

        public RequestManager(PaymentContext paymentContext)
        {
            _paymentContext = paymentContext ?? throw new ArgumentNullException(nameof(paymentContext));
        }
        
        public async Task<bool> ExistAsync(string tradeNo)
        {
            var request = await _paymentContext.Set<ClientRequest>().FirstOrDefaultAsync(c=>c.TradeNo.Equals(tradeNo));

            return request != null;
        }

        public async Task<ClientRequest> CreateRequestAsync(string tradeNo, ClientRequestType clientRequestType)
        {
            var request = new ClientRequest
            {
                    TradeNo = tradeNo,
                    Time = DateTime.Now,
                    ClientRequestType = clientRequestType
                };

            await _paymentContext.AddAsync(request);
            
            await _paymentContext.SaveChangesAsync();

            return request;
        }

        public async Task<bool> RemoveRequestAsync(string tradeNo)
        {
            var request = await  _paymentContext.Set<ClientRequest>().FirstOrDefaultAsync(c=>c.TradeNo.Equals(tradeNo));
            
            if (request ==null)
                return false;

            _paymentContext.Remove(request);

            return await _paymentContext.SaveChangesAsync() > 0;
        }

        public async Task<bool> UpdateRequestOrderRecordNoAsync(ClientRequest request,
            string responseData)
        {
            request.ResponseData = responseData;

            return await _paymentContext.SaveChangesAsync() > 0;
        }

        public async Task<ClientRequest> GetAsync(string tradeNo)
            => await _paymentContext.Set<ClientRequest>().FirstOrDefaultAsync(c=>c.TradeNo.Equals(tradeNo));
        
    }
}