using System.Linq;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using PXPlus.Payment.Domain.OrderRecordAggregate;
using PXPlus.Payment.Domain.SeedWork;

namespace PXPlus.Payment.Infrastructure.Repositories
{
    public class OrderRecordRepository : IOrderRecordRepository
    {
        private readonly PaymentContext _context;

        public IUnitOfWork UnitOfWork => _context;

        public OrderRecordRepository(PaymentContext context)
        {
            _context = context;
        }


        public async Task AddAsync(OrderRecord orderRecord)
            => await _context.OrderRecords.AddAsync(orderRecord);

        public async Task<OrderRecord> GetLastSuccessfulOrderRecordAsync(long paymentOrderId)
            => await _context.OrderRecords
                .AsNoTracking()
                .Where(c => c.OrderId.Equals(paymentOrderId))
                .OrderByDescending(c => c.Id)
                .FirstOrDefaultAsync();
        
    }
}