using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.SeedWork;

namespace PXPlus.Payment.Infrastructure.Repositories
{
    public class OrderRepository : IOrderRepository
    {
        private readonly PaymentContext _context;
        
        public IUnitOfWork UnitOfWork => _context;

        public OrderRepository(PaymentContext context)
        {
            _context = context;
        }

        public Task<Order> GetOrderByAsync(string orderNo)
            => _context.Orders.FirstOrDefaultAsync(c => c.OrderNo.SerialNumber.Equals(orderNo.Substring(2)));

        public Task<Order> GetOrderByNoTrackingAsync(string orderNo)
            => _context.Orders.AsNoTracking().FirstOrDefaultAsync(c => c.OrderNo.SerialNumber.Equals(orderNo.Substring(2)));
        

        public async Task AddAsync(Order order)
            => await _context.Orders.AddAsync(order);

        public void UnChangeOrder(Order order)
        {
            var entry = _context.Entry(order);
            entry.State = EntityState.Unchanged;
        }
    }
}