﻿using System.Text.Json;
using MediatR;
using PaymentService;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Application.Dto;
using Xunit;
using CancelPaymentGrpcService = PXPlus.Payment.Api.Services.CancelPaymentGrpcService;

namespace PXPlus.Payment.Api.Test.GrpcServiceMapTest
{
    public class CancelPaymentGrpcTest
    {
        private readonly CancelPaymentGrpcServiceTest _target;

        public CancelPaymentGrpcTest()
        {
            _target = new CancelPaymentGrpcServiceTest();
        }

        [Fact]
        public void CancelPaymentCommandMapFromCancelPaymentGrpcRequest()
        {
            var inputGrpc = new CancelPaymentGrpcRequest
            {
                PayerId = 7896,
                StoreCode = "77C822EA-5F1B-4D45-ABC3-17248429752F",
                TradeNo = "7A8AF1DC-F413-456B-B6A8-7F4BB0EF3375",
                ExecuteSource = 88,
                Executor = "CD04A8D7-72DA-4178-A017-68E631DF08A5",
                Cause = "B2B3E548-C674-494A-927F-743A06CF8C7B"
            };

            var excepted = new CancelPaymentCommand
            {
                PayerId = inputGrpc.PayerId,
                StoreCode = inputGrpc.StoreCode,
                TradeNo = inputGrpc.TradeNo,
                ExecuteSource = inputGrpc.ExecuteSource,
                Executor = inputGrpc.Executor,
                RefundCause = inputGrpc.Cause
            };

            var actual = _target.MapFromTest(inputGrpc);
            
            Assert.Equal(JsonSerializer.Serialize(excepted),JsonSerializer.Serialize(actual));
        }

        [Fact]
        public void ResponseMapToCancelPaymentGrpcResponse()
        {
            var response = new Response
            {
                Code = "57BE1894-A6A1-4E98-BC20-DEAA156C7891",
                Prefix = "6300F54B-53CC-4C00-B373-1D8E1DC65F65",
                Message = "2AF4E6B2-B56F-4149-A351-34C72812D3F7"
            };

            var excepted = new CancelPaymentGrpcResponse
            {
                Code = response.Code,
                PrefixCode = response.Prefix,
                Message = response.Message
            };

            var actual = _target.MapToTest(response);
            
            Assert.Equal(JsonSerializer.Serialize(excepted),JsonSerializer.Serialize(actual));
        }
        
        [Fact]
        public void CancelPaymentWithPxTradeNoCommandMapFromCancelPaymentGrpcWithPxTradeNoRequest()
        {
            var inputGrpc = new CancelPaymentWithPxTradeNoGrpcRequest()
            {
                PayerId = 7896,
                StoreCode = "77C822EA-5F1B-4D45-ABC3-17248429752F",
                TradeNo = "7A8AF1DC-F413-456B-B6A8-7F4BB0EF3375",
                PxTradeNo = "CF8D8BE3-1A50-4952-BAA3-1FDF747E5C6E",
                Cause = "33AEAAB1-7A96-4C0B-BFC8-E7933256E6BF",
                ExecuteSource = 4,
                Executor = "FC915847-61FF-4D4B-A4F6-8375E7F8F15E"
            };

            var excepted = new CancelPaymentWithPxTradeNoCommand
            {
                PayerId = inputGrpc.PayerId,
                StoreCode = inputGrpc.StoreCode,
                TradeNo = inputGrpc.TradeNo,
                PxTradeNo = inputGrpc.PxTradeNo,
                RefundCause = inputGrpc.Cause,
                ExecuteSource = inputGrpc.ExecuteSource,
                Executor = inputGrpc.Executor
            };

            var actual = _target.MapFromTest(inputGrpc);
          
            Assert.Equal(JsonSerializer.Serialize(excepted),JsonSerializer.Serialize(actual));
        }
        
        
        [Fact]
        public void ResponseMapToCancelPaymentWithPxTradeNoGrpcResponse()
        {
            var response = new Response
            {
                Code = "57BE1894-A6A1-4E98-BC20-DEAA156C78912",
                Prefix = "6300F54B-53CC-4C00-B373-1D8E1DC65F653",
                Message = "2AF4E6B2-B56F-4149-A351-34C72812D3F74"
            };

            var excepted = new CancelPaymentGrpcResponse
            {
                Code = response.Code,
                PrefixCode = response.Prefix,
                Message = response.Message
            };

            var actual = _target.CancelWithPxTradeNoMapToTest(response);
            
            Assert.Equal(JsonSerializer.Serialize(excepted),JsonSerializer.Serialize(actual));
        }

    }

    public class CancelPaymentGrpcServiceTest : CancelPaymentGrpcService
    {
        public CancelPaymentGrpcServiceTest() : base(null)
        {
        }

        public CancelPaymentCommand MapFromTest(CancelPaymentGrpcRequest request)
        {
            return MapFrom(request);
        }

        public CancelPaymentGrpcResponse MapToTest(Response response)
        {
            return MapTo(response);
        }

        public CancelPaymentWithPxTradeNoCommand MapFromTest(CancelPaymentWithPxTradeNoGrpcRequest request)
        {
            return MapFrom(request);
        }
        
        public CancelPaymentWithPxTradeNoGrpcResponse CancelWithPxTradeNoMapToTest(Response response)
        {
            return CancelWithPxTradeNoMapTo(response);
        }
    }
}