﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using Google.Protobuf.WellKnownTypes;
using PaymentService;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.OrderAggregate;
using Xunit;
using Activity = PXPlus.Payment.Application.Commands.Activity;

namespace PXPlus.Payment.Api.Test.GrpcServiceMapTest
{
    public class PaymentAppResMapToGrpcResTest
    {
        private readonly PaymentAppResMapToGrpcRes _target;

        public PaymentAppResMapToGrpcResTest()
        {
            _target = new PaymentAppResMapToGrpcRes();
        }

        [Fact]
        public void PaymentAppResponseMapToPaymentGrpcResponse()
        {
            
           var activities = new List<Activity>
                {
                    new Activity
                    {
                        ActivityCode = "EEE97B54-CD48-4E59-9447-ABD0272D7694",
                        ProductAmount = 555
                    },
                    new Activity
                    {
                        ActivityCode = "F2AECFC9-1FB9-4926-8770-0ADBE54DF9CC",
                        ProductAmount = 666
                    }
                }; 
           
            var paymentAppRes = new PaymentAppResponse
            {
                Code = "3526BF8A-03ED-40AE-8962-8F6D939DDCCB",
                Message = "724F0215-482D-4D17-B925-2B200E4DF27A",
                Prefix = "4723DA0A-D2B6-46A1-BD9D-724198A54C49",
                PXTradeNo = "9CE20FAF-5AB0-4B47-BBDA-E411C1553792",
                OrderAmount = 123,
                PayAmount = 456,
                DiscountPoint = 789,
                PayToolType = PayToolType.EPAccount,
                PayToolName = "DC6B119E-B680-4D1F-9EBE-E62D95147F35",
                PayToolIdentity = "28EC567B-B25A-484D-B5F2-0F2F226E9E6B",
                NoneDiscountAmount = 753,
                NoneFeedbackAmount = 214,
                TransactionTime = DateTime.Now.AddDays(15),
                PayerEpAccount = "E657327B-80AD-4850-846A-15AFEE8C9E33",
                StoreName = "5E305F19-E6EA-4CFB-9AAC-3313A5E6AD4E",
                StoreEpAccount = "658B6145-07FB-4FAA-B12E-55285461A21A",
                MerchantName = "CC6BD01F-DF4D-4688-85BA-019C324C3A44",
                Activities = activities,
                PayerId = 999
            };

            var excepted = new PaymentGrpcResponse
            {
                Code = paymentAppRes.Code,
                Message = paymentAppRes.Message,
                PrefixCode = paymentAppRes.Prefix,
                Data = new PaymentInfo
                {
                    PxTradeNo = paymentAppRes.PXTradeNo,
                    OrderAmount = (double)paymentAppRes.OrderAmount,
                    PayAmount = (double)paymentAppRes.PayAmount,
                    DiscountPoint = (double)paymentAppRes.DiscountPoint,
                    PayToolType = (int)paymentAppRes.PayToolType,
                    PayToolName = paymentAppRes.PayToolName,
                    PayToolIdentity = paymentAppRes.PayToolIdentity,
                    NoneDiscountAmount = (double)paymentAppRes.NoneDiscountAmount,
                    NoneFeedbackAmount = (double)paymentAppRes.NoneFeedbackAmount,
                    TransactionTime = paymentAppRes.TransactionTime.ToUniversalTime().ToTimestamp(),
                    PayerEpAccount = paymentAppRes.PayerEpAccount,
                    StoreName =paymentAppRes.StoreName,
                    StoreEpAccount = paymentAppRes.StoreEpAccount,
                    MerchantName = paymentAppRes.MerchantName,
                    PayerId = paymentAppRes.PayerId
                }
            };

            excepted.Data.Activities.AddRange(activities.Select(c=>new ActivityDiscount
            {
                ActivityCode = c.ActivityCode,
                DiscountAmount = c.DiscountAmount,
                ProductAmount = c.ProductAmount
            }));

            var actual = _target.MapTo(paymentAppRes);
            
            Assert.Equal(JsonSerializer.Serialize(excepted),JsonSerializer.Serialize(actual));
        }
    }
}