﻿using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using PaymentService;
using PXPlus.Enum.Authorization;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Domain.OrderAggregate;
using Xunit;
using Activity = PaymentService.Activity;
using MetaData = PaymentService.MetaData;
using PaymentGrpcService = PXPlus.Payment.Api.Services.PaymentGrpcService;

namespace PXPlus.Payment.Api.Test.GrpcServiceMapTest
{
    public class PaymentGrpcTest
    {
        private readonly PaymentGrpcServiceTest _target;

        public PaymentGrpcTest()
        {
            _target = new PaymentGrpcServiceTest();
        }

        [Fact]
        public void PaymentCommandMapFromPaymentGrpcRequest()
        {
            var grpcRequest = new PaymentGrpcRequest
            {
                TradeNo = "iamtradeno",
                PayerId = 111,
                PayToolSeq = 1,
                PayToolType = 3,
                IsUsePoint = true,
                StoreCode = "iamstorecode",
                Amount = 666,
                NoneDiscountAmount = 777,
                NoneFeedbackAmount = 888,
                PlatformType = 2,
                IsEcTrade = true,
                MerchantTradeNo = "iammerchanttradeno",
                MemberPhoneNo = "iammemberphoneno",
                MetaData = new MetaData()
                {
                    Type = 2,
                    Content = "HelloContent"
                }
            };

            var activities = new List<Activity>
            {
                new Activity
                {
                    ActivityCode = "iamactivitycode1",
                    ProductAmount = 123
                },
                new Activity
                {
                    ActivityCode = "iamactivitycode2",
                    ProductAmount = 456
                }

            }; 
            
            grpcRequest.Activities.AddRange(activities);


            var excepted = new PaymentCommand((decimal)grpcRequest.Amount,grpcRequest.PayerId,grpcRequest.StoreCode
            ,grpcRequest.PayToolSeq,grpcRequest.IsUsePoint,(decimal)grpcRequest.NoneFeedbackAmount,(decimal)grpcRequest.NoneDiscountAmount
            ,(PayToolType)grpcRequest.PayToolType,(PlatformType)grpcRequest.PlatformType,activities.Select(c=>new ActivityCashDiscountInfo
            {
                ProductAmount = (decimal)c.ProductAmount,
                ActivityId = c.ActivityCode
            }),grpcRequest.IsEcTrade,grpcRequest.MemberPhoneNo,grpcRequest.MerchantTradeNo,
            new Domain.OrderAggregate.MetaData
            {
                Type = grpcRequest.MetaData.Type,
                Content = grpcRequest.MetaData.Content
            }
            );

            var actual = _target.MapFromTest(grpcRequest);
            
            Assert.Equal(JsonSerializer.Serialize(excepted),JsonSerializer.Serialize(actual));
        }
    }

    public class PaymentGrpcServiceTest : PaymentGrpcService
    {
        public PaymentGrpcServiceTest() : base(null, null)
        {
        }

        public PaymentCommand MapFromTest(PaymentGrpcRequest request)
        {
            return MapFrom(request);
        }
    }
}