﻿using System;
using System.Collections.Generic;
using System.Linq;
using PaymentService;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Application.Dto;
using Xunit;
using Activity = PXPlus.Payment.Application.Commands.Activity;


namespace PXPlus.Payment.Api.Test.GrpcServiceMapTest
{
    public class RefundAppResMapToGrpcResTest
    {
        private readonly RefundAppResMapToGrpcRes _target;

        public RefundAppResMapToGrpcResTest()
        {
            _target = new RefundAppResMapToGrpcRes();
        }

        [Fact]
        public void RefundGrpcResponseMapToRefundAppResponse()
        {
            var activities = new List<Activity>
            {
                new Activity
                {
                    ActivityCode = "iamactivitycode1",
                    ProductAmount = 123
                },
                new Activity
                {
                    ActivityCode = "iamactivitycode2",
                    ProductAmount = 456
                }

            }; 
            
            var response = new RefundAppResponse
            {
                Code = "87C26454-896E-46DB-B150-1A0E6FD64D94",
                Message = "208348B4-C77A-465E-913F-D6472F1ECD3B",
                Prefix = "8F91F98B-2A7B-41FE-86DF-68267BB67355",
                PXTradeNo = "2F7548B2-A810-47B1-85B6-154B65F5BF1E",
                Amount = 789,
                TradeAmount = 63,
                DiscountPoint = 321,
                Activities = activities
            };

            var excepted = new RefundGrpcResponse
            {
                Code = response.Code,
                Message = response.Message,
                PrefixCode = response.Prefix,
                Data = new RefundInfo
                {
                    PxTradeNo = response.PXTradeNo,
                    Amount = (double)response.Amount,
                    TradeAmount = (double)response.TradeAmount,
                    DiscountPoint = (double)response.DiscountPoint
                }
            };
            
            excepted.Data.Activities.AddRange(activities.Select(c=>new ActivityDiscount
            {
                ActivityCode = c.ActivityCode,
                DiscountAmount = c.DiscountAmount,
                ProductAmount = c.ProductAmount
            }));

            var actual = _target.MapTo(response);
            
            Assert.Equal(excepted.Code,actual.Code);
            Assert.Equal(excepted.Message,actual.Message);
            Assert.Equal(excepted.PrefixCode,actual.PrefixCode);
            Assert.Equal(excepted.Data.PxTradeNo,actual.Data.PxTradeNo);
            Assert.Equal(excepted.Data.Amount,actual.Data.Amount);
            Assert.Equal(excepted.Data.TradeAmount,actual.Data.TradeAmount);
            Assert.Equal(excepted.Data.DiscountPoint,actual.Data.DiscountPoint);
            
            Assert.Equal(excepted.Data.Activities.Count, actual.Data.Activities.Count);
            
            var zip = excepted.Data.Activities.Join(actual.Data.Activities,
                e => e.ActivityCode, a => a.ActivityCode, (e, a) => new
                {
                    e.ActivityCode,
                    ExceptedAmount = e.ProductAmount,
                    ActualAmount = a.ProductAmount
                });
            
            foreach (var exceptedData in zip) 
                Assert.Equal(exceptedData.ExceptedAmount,exceptedData.ActualAmount);
        }
    }
}