﻿using System.Collections.Generic;
using System.Linq;
using CreditCardPaymentService;
using MediatR;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Domain.OrderAggregate;
using Xunit;
using RefundGrpcRequest = PaymentService.RefundGrpcRequest;
using RefundGrpcService = PXPlus.Payment.Api.Services.RefundGrpcService;

namespace PXPlus.Payment.Api.Test.GrpcServiceMapTest
{
    public class RefundGrpcTest
    {
        private readonly RefundGrpcServiceTest _target;

        public RefundGrpcTest()
        {
            _target = new RefundGrpcServiceTest();
        }

        [Fact]
        public void RefundGrpcRequestMapToRefundCommand()
        {
            var activities = new List<Activity>
                {
                    new Activity
                    {
                        ActivityCode = "EEE97B54-CD48-4E59-9447-ABD0272D7694",
                        ProductAmount = 555
                    },
                    new Activity
                    {
                        ActivityCode = "F2AECFC9-1FB9-4926-8770-0ADBE54DF9CC",
                        ProductAmount = 666
                    }
                }; 
                
            var inputGrpc = new RefundGrpcRequest
            {
                TradeNo = "C1D0210A-F802-4780-B99F-F70978D39590",
                RefunderId = 123,
                StoreCode = "239B5160-403E-4C79-ACBA-62E0037B8F82",
                OriPxTradeNo = "4179F1C9-14EC-4E7A-9304-1160D91CB644",
                Amount = 4,
                NoneFeedbackAmount = 96,
                NoneDiscountAmount = 5,
                MerchantTradeNo = "ABEFEA94-EFE9-45CC-B04F-730D608A7BAF",
                Cause = "15C685BE-6CB8-4FEA-AB8F-BB4A96C61722",
                ExecuteSource = 7,
                Executor = "D20FA434-6DFB-4FFA-B092-79F697EB34C5"
            };

            inputGrpc.Activities.AddRange(activities.Select(c=>new PaymentService.Activity
            {
                ActivityCode = c.ActivityCode,
                ProductAmount = c.ProductAmount
            }));

            var excepted = new RefundCommand( inputGrpc.RefunderId,
                inputGrpc.StoreCode,
                inputGrpc.OriPxTradeNo,
                (decimal) inputGrpc.Amount,
                (decimal) inputGrpc.NoneFeedbackAmount,
                (decimal) inputGrpc.NoneDiscountAmount,
                activities.Select(c=>new ActivityCashDiscountInfo
                {
                    ActivityId = c.ActivityCode,
                    ProductAmount = (decimal)c.ProductAmount
                }),
                inputGrpc.MerchantTradeNo,
                inputGrpc.Cause,
                inputGrpc.ExecuteSource,
                inputGrpc.Executor);

            var actual = _target.MapToTest(inputGrpc);
            
            Assert.Equal(excepted.StoreCode,actual.StoreCode);
            Assert.Equal(excepted.OriginalPxTradeNo,actual.OriginalPxTradeNo);
            Assert.Equal(excepted.Amount,actual.Amount);
            Assert.Equal(excepted.NoneFeedbackAmount,actual.NoneFeedbackAmount);
            Assert.Equal(excepted.NoneDiscountAmount,actual.NoneDiscountAmount);
            Assert.Equal(excepted.ActivityCashDiscountInfos.Count(),actual.ActivityCashDiscountInfos.Count());
            Assert.Equal(excepted.RefundCause,actual.RefundCause);
            Assert.Equal(excepted.Executor,actual.Executor);
            Assert.Equal(excepted.ExecuteSource,actual.ExecuteSource);

            var zip = excepted.ActivityCashDiscountInfos.Join(actual.ActivityCashDiscountInfos,
                e => e.ActivityId, a => a.ActivityId, (e, a) => new
                {
                    e.ActivityId,
                    ExceptedAmount = e.ProductAmount,
                    ActualAmount = a.ProductAmount
                });
            
            foreach (var exceptedData in zip) 
                Assert.Equal(exceptedData.ExceptedAmount,exceptedData.ActualAmount);

        }
    }
    
    public class RefundGrpcServiceTest : RefundGrpcService
    {
        public RefundGrpcServiceTest() : base(null,null)
        {
        }

        public RefundCommand MapToTest(RefundGrpcRequest request)
        {
            return MapTo(request);
        }
    }
}