using System;
using System.Threading.Tasks;
using AutoMapper;
using Grpc.Core;
using Grpc.Core.Interceptors;
using Microsoft.Extensions.Logging;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.DomainExceptions;

namespace PXPlus.Payment.Api.Infrastructure.Interceptors
{
    public class GrpcGlobalExceptionHandleInterceptor : Interceptor
    {
        private readonly IMapper _mapper;
        private readonly ILogger<GrpcGlobalExceptionHandleInterceptor> _logger;

        public GrpcGlobalExceptionHandleInterceptor(IMapper mapper, ILogger<GrpcGlobalExceptionHandleInterceptor> logger)
        {
            _mapper = mapper;
            _logger = logger;
        }

        public override async Task<TResponse> UnaryServerHandler<TRequest, TResponse>(TRequest request,
            ServerCallContext context,
            UnaryServerMethod<TRequest, TResponse> continuation)
        {
            try
            {
                return await continuation(request, context);
            }
            catch (OrderDomainException de)
            {
                var response = new Response
                               {Code = de.ResultCode, Message = de.ResultMessage + de.AdditionalMessage};
                
                return _mapper.Map<TResponse>(response);
            }
            catch (Exception e)
            {
                var response = new Response {Code = ((int)PaymentStatusCode.OtherError).ToString()
                                                , Message = PaymentStatusCode.OtherError + e.Message};
                
                _logger.LogError(e,e.Message);
                
                return _mapper.Map<TResponse>(response);
            }
        }
    }
}