using System.Linq;
using Google.Protobuf.WellKnownTypes;
using PaymentService;
using PXPlus.Payment.Application.Dto;

namespace PXPlus.Payment.Api.Infrastructure.Mapper
{
    public class PaymentAppResMapToGrpcRes
    {
        public PaymentGrpcResponse MapTo(PaymentAppResponse response)
        {
            var grpcResponse =
                new PaymentGrpcResponse
                {
                    Code = response.Code ?? "",
                    Message = response.Message ?? "",
                    PrefixCode = response.Prefix ?? "",
                    Data = new PaymentInfo
                           {
                               PxTradeNo = response.PXTradeNo ?? "",
                               OrderAmount = (double) response.OrderAmount,
                               PayAmount= (double) response.PayAmount,
                               DiscountPoint = (double) response.DiscountPoint,
                               PayToolType = (int)response.PayToolType ,
                               PayToolName = response.PayToolName ?? "",
                               PayToolIdentity = response.PayToolIdentity ?? "",
                               NoneDiscountAmount = (double) response.NoneDiscountAmount,
                               NoneFeedbackAmount = (double) response.NoneFeedbackAmount,
                               TransactionTime = Timestamp.FromDateTime(response.TransactionTime.ToUniversalTime()),
                               PayerEpAccount = response.PayerEpAccount ?? "",
                               StoreName = response.StoreName ?? "",
                               StoreEpAccount = response.StoreEpAccount ?? "",
                               MerchantName = response.MerchantName ?? "",
                               PayerId = response.PayerId
                           }
                };

            if (response.Activities != null && response.Activities.Any())
                grpcResponse.Data.Activities.AddRange(response.Activities.Select(s => new ActivityDiscount
                        {
                            ActivityCode = s.ActivityCode,
                            ProductAmount = s.ProductAmount,
                            DiscountAmount = s.DiscountAmount,
                        })
                );

            return grpcResponse;
        }
    }
}