﻿using System.Threading.Tasks;
using Grpc.Core;
using MediatR;
using PaymentService;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Application.Dto;

namespace PXPlus.Payment.Api.Services
{
    public class CancelPaymentGrpcService : PaymentService.CancelPaymentGrpcService.CancelPaymentGrpcServiceBase
    {
        private readonly IMediator _mediator;

        public CancelPaymentGrpcService(IMediator mediator)
        {
            _mediator = mediator;
        }

        public override async Task<CancelPaymentGrpcResponse> CancelPaymentCommand(CancelPaymentGrpcRequest request, ServerCallContext context)
        {
            var command = MapFrom(request);
            
            var response = await _mediator.Send(command);
            
            return MapTo(response);
        }


        protected CancelPaymentCommand MapFrom(CancelPaymentGrpcRequest request)
            =>
                new()
                {
                    TradeNo = request.TradeNo,
                    PayerId = request.PayerId,
                    StoreCode = request.StoreCode,
                    RefundCause = request.Cause,
                    ExecuteSource = request.ExecuteSource,
                    Executor = request.Executor
                };

        protected CancelPaymentGrpcResponse MapTo(Response response)
            => new()
            {
                Code = response.Code,
                Message = response.Message,
                PrefixCode = response.Prefix
            };
        
        
        
        public override async Task<CancelPaymentWithPxTradeNoGrpcResponse> CancelPaymentWithPxTradeNoCommand(CancelPaymentWithPxTradeNoGrpcRequest request, ServerCallContext context)
        {
            var command = MapFrom(request);
            
            var identifierCommand = new IdentifiedCommand<CancelPaymentWithPxTradeNoCommand, Response>(command,request.TradeNo);
                
            var response = await _mediator.Send(identifierCommand);

            return CancelWithPxTradeNoMapTo(response);
        }

        protected CancelPaymentWithPxTradeNoCommand MapFrom(CancelPaymentWithPxTradeNoGrpcRequest request)
            => new()
            {
                TradeNo = request.TradeNo,
                PxTradeNo = request.PxTradeNo,
                PayerId = request.PayerId,
                StoreCode = request.StoreCode,
                RefundCause = request.Cause,
                ExecuteSource = request.ExecuteSource,
                Executor = request.Executor
            };
        
        protected CancelPaymentWithPxTradeNoGrpcResponse CancelWithPxTradeNoMapTo(Response response)
            => new()
            {
                Code = response.Code,
                Message = response.Message,
                PrefixCode = response.Prefix
            };
    }
}