﻿using System.Threading.Tasks;
using Grpc.Core;
using MediatR;
using PaymentService;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Application.Dto;

namespace PXPlus.Payment.Api.Services
{
    public class ConfirmOrderGrpcService : OrderConfirmGrpcService.OrderConfirmGrpcServiceBase
    {
        private readonly IMediator _mediator;
        private readonly PaymentAppResMapToGrpcRes _paymentAppResMapToGrpcRes;

        public ConfirmOrderGrpcService(IMediator mediator, PaymentAppResMapToGrpcRes paymentAppResMapToGrpcRes)
        {
            _mediator = mediator;
            _paymentAppResMapToGrpcRes = paymentAppResMapToGrpcRes;
        }

        public override async Task<PaymentGrpcResponse> OrderConfirmCommand(OrderConfirmGrpcRequest request,
            ServerCallContext context)
        {
            var command = MapFrom(request);

            var identifierCommand =
                new IdentifiedCommand<OrderConfirmCommand, PaymentAppResponse>(command, request.TradeNo);

            var response = await _mediator.Send(identifierCommand);

            return _paymentAppResMapToGrpcRes.MapTo(response);
        }

        protected OrderConfirmCommand MapFrom(OrderConfirmGrpcRequest request)
            => new(
                request.PxTradeNo,
                request.PayerId,
                request.StoreCode);
    }
}