﻿using System.Linq;
using System.Threading.Tasks;
using Grpc.Core;
using MediatR;
using PaymentService;
using PXPlus.Enum.Authorization;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Application.Dto;
using MetaData = PXPlus.Payment.Domain.OrderAggregate.MetaData;

namespace PXPlus.Payment.Api.Services
{
    public class PaymentGrpcService : PaymentService.PaymentGrpcService.PaymentGrpcServiceBase
    {
        private readonly IMediator _mediator;

        private readonly PaymentAppResMapToGrpcRes _paymentAppResMapToGrpcRes;

        public PaymentGrpcService(IMediator mediator, PaymentAppResMapToGrpcRes paymentAppResMapToGrpcRes)
        {
            _mediator = mediator;
            _paymentAppResMapToGrpcRes = paymentAppResMapToGrpcRes;
        }

        public override async Task<PaymentGrpcResponse> PaymentCommand(PaymentGrpcRequest request, ServerCallContext context)
        {
            var paymentCommand = MapFrom(request);

            var identifierCommand = new IdentifiedCommand<PaymentCommand, PaymentAppResponse>(paymentCommand,request.TradeNo);
            
            var response = await _mediator.Send(identifierCommand);
            
            return _paymentAppResMapToGrpcRes.MapTo(response);
        }

        protected PaymentCommand MapFrom(PaymentGrpcRequest request)
            => new ((decimal) request.Amount,
                    request.PayerId,
                    request.StoreCode,
                    request.PayToolSeq,
                    request.IsUsePoint,
                    (decimal) request.NoneFeedbackAmount,
                    (decimal) request.NoneDiscountAmount,
                    (PayToolType) request.PayToolType,
                    (PlatformType)request.PlatformType,
                    request.Activities.Select(s =>
                        new ActivityCashDiscountInfo(s.ActivityCode, (decimal) s.ProductAmount)),
                    request.IsEcTrade,
                    request.MemberPhoneNo,
                    request.MerchantTradeNo,
                    new MetaData
                    {
                        Type = request.MetaData?.Type ?? 0,
                        Content = request.MetaData?.Content
                    }
                );
    }
}