﻿using System.Threading.Tasks;
using Grpc.Core;
using PaymentService;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Application.Queries;

namespace PXPlus.Payment.Api.Services
{
    public class QueryGrpcService : PaymentService.QueryGrpcService.QueryGrpcServiceBase
    {
        private readonly QueryAppService _queryAppService;
        private readonly PaymentAppResMapToGrpcRes _paymentAppResMapToGrpcRes;
        private readonly RefundAppResMapToGrpcRes _refundAppResMapToGrpcRes;

        public QueryGrpcService(QueryAppService queryAppService, PaymentAppResMapToGrpcRes paymentAppResMapToGrpcRes, RefundAppResMapToGrpcRes refundAppResMapToGrpcRes)
        {
            _queryAppService = queryAppService;
            _paymentAppResMapToGrpcRes = paymentAppResMapToGrpcRes;
            _refundAppResMapToGrpcRes = refundAppResMapToGrpcRes;
        }
        
        public override async Task<QueryGrpcResponse> Query(QueryGrpcRequest request, ServerCallContext context)
        {
            var result = await _queryAppService.QueryAsync(request.TradeNo);
            return To(result);
        }

        private QueryGrpcResponse To(QueryAppResponse response)
            => new()
               {
                   Code = "0000",
                   PrefixCode = "PS",
                   Message = "Success",
                   Data = new QueryGrpcResponse.Types.Data
                          {
                              OrderType = (QueryOrderType) response.OrderQueryType,
                              PaymentInfo = response.PaymentAppResponse ==null ? null :_paymentAppResMapToGrpcRes.MapTo(response.PaymentAppResponse),
                              RefundInfo = response.RefundAppResponse == null ? null : _refundAppResMapToGrpcRes.MapTo(response.RefundAppResponse)
                          }
               };

        public override async Task<QueryGrpcResponse> QueryWithPxTradeNo(QueryWithPxTradeNoGrpcRequest request, ServerCallContext context)
        {
            var response = await _queryAppService.QueryWithPxTradeNoAsync(request.PxTradeNo);

            return To(response);
        }
    }
}