﻿using System.Linq;
using System.Threading.Tasks;
using Grpc.Core;
using MediatR;
using PaymentService;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Application.Dto;

namespace PXPlus.Payment.Api.Services
{
    public class RefundGrpcService : global::PaymentService.RefundGrpcService.RefundGrpcServiceBase
    {
        private readonly IMediator _mediator;
        private readonly RefundAppResMapToGrpcRes _refundAppResMapToGrpcRes;

        public RefundGrpcService(IMediator mediator, RefundAppResMapToGrpcRes refundAppResMapToGrpcRes)
        {
            _mediator = mediator;
            _refundAppResMapToGrpcRes = refundAppResMapToGrpcRes;
        }

        public override async Task<RefundGrpcResponse> RefundCommand(RefundGrpcRequest request,
            ServerCallContext context)
        {
            var command = MapTo(request);

            var identifierCommand = new IdentifiedCommand<RefundCommand, RefundAppResponse>(command, request.TradeNo);

            var result = await _mediator.Send(identifierCommand);

            return _refundAppResMapToGrpcRes.MapTo(result);
        }

        protected RefundCommand MapTo(RefundGrpcRequest request)
            => new(
                request.RefunderId,
                request.StoreCode,
                request.OriPxTradeNo,
                (decimal) request.Amount,
                (decimal) request.NoneFeedbackAmount,
                (decimal) request.NoneDiscountAmount,
                request.Activities.Select(s => new ActivityCashDiscountInfo(s.ActivityCode, (decimal) s.ProductAmount)),
                request.MerchantTradeNo,
                request.Cause,
                request.ExecuteSource,
                request.Executor);
    }
}